from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.gcp_base_helpers import gcp_alert_context
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk

gcp_cloud_run_service_created_tests: list[RuleTest] = [
    RuleTest(
        name="GCP No methodName found",
        expected_result=False,
        log={
            "p_event_time": "2024-07-22 14:20:56.237323088",
            "p_log_type": "GCP.AuditLog",
            "insertId": "123456789xyz",
            "logName": "projects/internal-sentry/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {"principalEmail": "0000000000000@cloudbuild.gserviceaccount.com"},
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "cloudbuild.builds.create",
                        "resource": "projects/00000000aaaaaaaa",
                        "resourceAttributes": {},
                    },
                ],
                "requestMetadata": {
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2024-07-22T14:20:55.898367039Z"},
                },
            },
            "receiveTimestamp": "2024-07-22 14:20:56.428021476",
            "resource": {
                "labels": {"method": "", "project_id": "some-project", "service": ""},
                "type": "audited_resource",
            },
            "severity": "NOTICE",
            "timestamp": "2024-07-22 14:20:56.237323088",
        },
    ),
    RuleTest(
        name="GCP Run Service Created",
        expected_result=True,
        log={
            "insertId": "jzm5rucrn2",
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "run.services.create",
                        "resource": "namespaces/some-project/services/cloudrun-exfil",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.cloud.run.v1.Services.CreateService",
                "request": {
                    "@type": "type.googleapis.com/google.cloud.run.v1.CreateServiceRequest",
                    "parent": "namespaces/some-project",
                    "service": {
                        "apiVersion": "serving.knative.dev/v1",
                        "kind": "Service",
                        "metadata": {
                            "annotations": {
                                "client.knative.dev/user-image": "us-west1-docker.pkg.dev/some-project/abc-test/run_services_create_test",
                            },
                            "name": "cloudrun-exfil",
                            "namespace": "some-project",
                        },
                        "spec": {
                            "template": {
                                "metadata": {
                                    "annotations": {
                                        "client.knative.dev/user-image": "us-west1-docker.pkg.dev/some-project/abc-test/run_services_create_test",
                                    },
                                    "labels": {"cloud.googleapis.com/location": "us-west1"},
                                    "name": "cloudrun-exfil-00001-zif",
                                },
                                "spec": {"serviceAccountName": "abc-test@some-project.iam.gserviceaccount.com"},
                            },
                        },
                    },
                },
                "requestMetadata": {
                    "callerIP": "1.2.3.4",
                    "callerSuppliedUserAgent": "(gzip),gzip(gfe)",
                    "destinationAttributes": {},
                    "requestAttributes": {"auth": {}, "time": "2024-02-02T09:43:54.690161Z"},
                },
                "resourceLocation": {"currentLocations": ["us-west1"]},
                "resourceName": "namespaces/some-project/services/cloudrun-exfil",
                "response": {
                    "@type": "type.googleapis.com/google.cloud.run.v1.Service",
                    "apiVersion": "serving.knative.dev/v1",
                    "kind": "Service",
                    "metadata": {
                        "annotations": {
                            "client.knative.dev/user-image": "us-west1-docker.pkg.dev/some-project/abc-test/run_services_create_test",
                            "run.googleapis.com/ingress": "all",
                            "run.googleapis.com/operation-id": "6fdf115a-1bdd-4836-b0ca-ae71f8ba6718",
                            "serving.knative.dev/creator": "some.user@company.com",
                            "serving.knative.dev/lastModifier": "some.user@company.com",
                        },
                        "creationTimestamp": "2024-02-02T09:43:54.640837Z",
                        "generation": 1,
                        "labels": {"cloud.googleapis.com/location": "us-west1"},
                        "name": "cloudrun-exfil",
                        "namespace": "1028347275902",
                        "resourceVersion": "AAYQYvNHUcU",
                        "selfLink": "/apis/serving.knative.dev/v1/namespaces/1028347275902/services/cloudrun-exfil",
                        "uid": "45101e8e-7b91-4c41-81a1-969e876923f4",
                    },
                    "spec": {
                        "template": {
                            "metadata": {
                                "annotations": {
                                    "autoscaling.knative.dev/maxScale": "100",
                                    "client.knative.dev/user-image": "us-west1-docker.pkg.dev/some-project/abc-test/run_services_create_test",
                                },
                                "labels": {"run.googleapis.com/startupProbeType": "Default"},
                                "name": "cloudrun-exfil-00001-zif",
                            },
                            "spec": {
                                "containerConcurrency": 80,
                                "serviceAccountName": "abc-test@some-project.iam.gserviceaccount.com",
                                "timeoutSeconds": 300,
                            },
                        },
                        "traffic": [{"latestRevision": True, "percent": 100}],
                    },
                    "status": {},
                },
                "serviceName": "run.googleapis.com",
            },
            "receiveTimestamp": "2024-02-02 09:43:54.723840817",
            "resource": {
                "labels": {
                    "configuration_name": "",
                    "location": "us-west1",
                    "project_id": "some-project",
                    "revision_name": "",
                    "service_name": "cloudrun-exfil",
                },
                "type": "cloud_run_revision",
            },
            "severity": "NOTICE",
            "timestamp": "2024-02-02 09:43:54.497796000",
        },
    ),
    RuleTest(
        name="GCP Run Service Not Created",
        expected_result=False,
        log={
            "insertId": "jfca5rd3wqn",
            "logName": "projects/some-project/logs/cloudaudit.googleapis.com%2Factivity",
            "protoPayload": {
                "at_sign_type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "authenticationInfo": {
                    "principalEmail": "some.user@company.com",
                    "principalSubject": "user:some.user@company.com",
                },
                "authorizationInfo": [
                    {
                        "granted": True,
                        "permission": "run.services.create",
                        "resource": "namespaces/some-project/services/cloudrun-exfil",
                        "resourceAttributes": {},
                    },
                ],
                "methodName": "google.cloud.run.v1.Services.CreateService",
                "request": "...",
                "requestMetadata": "...",
                "resourceLocation": "...",
                "resourceName": "...",
                "serviceName": "run.googleapis.com",
                "status": {"code": 6, "message": "Resource 'cloudrun-exfil' already exists."},
            },
            "receiveTimestamp": "2024-02-02 09:36:22.250430490",
            "resource": "...",
            "severity": "ERROR",
            "timestamp": "2024-02-02 09:36:21.212182000",
        },
    ),
]


@panther_managed
class GCPCloudRunServiceCreated(Rule):
    log_types = [LogType.GCP_AUDIT_LOG]
    default_description = "Detects creation of new Cloud Run Service, which, if configured maliciously, may be part of the attack aimed to invoke the service and retrieve the access token."
    display_name = "GCP Cloud Run Service Created"
    id = "GCP.Cloud.Run.Service.Created-prototype"
    create_alert = False
    default_reference = "https://cloud.google.com/run/docs/quickstarts/deploy-container"
    default_runbook = "Confirm this was authorized and necessary behavior"
    default_severity = Severity.LOW
    tests = gcp_cloud_run_service_created_tests

    def rule(self, event):
        if deep_get(event, "severity") == "ERROR":
            return False
        method_name = deep_get(event, "protoPayload", "methodName", default="")
        if not method_name.endswith("Services.CreateService"):
            return False
        authorization_info = deep_walk(event, "protoPayload", "authorizationInfo")
        if not authorization_info:
            return False
        for auth in authorization_info:
            if auth.get("permission") == "run.services.create" and auth.get("granted") is True:
                return True
        return False

    def title(self, event):
        actor = deep_get(event, "protoPayload", "authenticationInfo", "principalEmail", default="<ACTOR_NOT_FOUND>")
        project_id = deep_get(event, "resource", "labels", "project_id", default="<PROJECT_NOT_FOUND>")
        return f"[GCP]: [{actor}] created new Run Service in project [{project_id}]"

    def alert_context(self, event):
        context = gcp_alert_context(event)
        context["service_account"] = deep_get(
            event,
            "protoPayload",
            "request",
            "service",
            "spec",
            "template",
            "spec",
            default="<SERVICE_ACCOUNT_NOT_FOUND>",
        )
        return context



import json
from unittest.mock import MagicMock

from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_azuresignin_helpers import actor_user, azure_signin_alert_context, is_sign_in_event
from pypanther.helpers.panther_base_helpers import deep_get

azure_audit_legacy_auth_tests: list[RuleTest] = [
    RuleTest(
        name="Failed Sign-In",
        expected_result=False,
        log={
            "calleripaddress": "12.12.12.12",
            "category": "ServicePrincipalSignInLogs",
            "correlationid": "e1f237ef-6548-4172-be79-03818c04c06e",
            "durationms": 0,
            "level": 4,
            "location": "IE",
            "operationname": "Sign-in activity",
            "operationversion": 1,
            "p_event_time": "2023-07-26 23:00:20.889",
            "p_log_type": "Azure.Audit",
            "properties": {
                "appId": "cfceb902-8fab-4f8c-88ba-374d3c975c3a",
                "authenticationProcessingDetails": [{"key": "Azure AD App Authentication Library", "value": ""}],
                "authenticationProtocol": "none",
                "clientCredentialType": "none",
                "conditionalAccessStatus": "notApplied",
                "correlationId": "5889315c-c4ac-4807-99da-e17417eae786",
                "createdDateTime": "2023-07-26 22:58:30.983201900",
                "crossTenantAccessType": "none",
                "flaggedForReview": False,
                "id": "36658c78-02d9-4d8f-84ee-5ca4a3fdefef",
                "incomingTokenType": "none",
                "ipAddress": "12.12.12.12",
                "isInteractive": False,
                "isTenantRestricted": False,
                "location": {
                    "city": "Dublin",
                    "countryOrRegion": "IE",
                    "geoCoordinates": {"latitude": 51.35555555555555, "longitude": -5.244444444444444},
                    "state": "Dublin",
                },
                "managedIdentityType": "none",
                "processingTimeInMilliseconds": 0,
                "resourceDisplayName": "Azure Storage",
                "resourceId": "037694de-8c7d-498d-917d-edb650090fa5",
                "resourceServicePrincipalId": "a225221f-8cc5-411a-9cc7-5e1394b8a5b8",
                "riskDetail": "none",
                "riskLevelAggregated": "low",
                "riskLevelDuringSignIn": "low",
                "riskState": "none",
                "servicePrincipalId": "b1c34143-e405-4058-8e29-84596ad737b8",
                "servicePrincipalName": "some-service-principal",
                "status": {"errorCode": 7000215},
                "tokenIssuerType": "AzureAD",
                "uniqueTokenIdentifier": "NDDDDDDDDDDDDDDDDDD_DD",
            },
            "resourceid": "/tenants/c0dd2fa0-71be-4df8-b2a6-24cee7de069a/providers/Microsoft.aadiam",
            "resultsignature": "None",
            "resulttype": 7000215,
            "tenantid": "a2aa49aa-2c0c-49d2-af87-f402c421df0b",
            "time": "2023-07-26 23:00:20.889",
        },
    ),
    RuleTest(
        name="Sign-In with legacy auth",
        expected_result=True,
        log={
            "calleripaddress": "173.53.70.163",
            "category": "SignInLogs",
            "correlationid": "1043d312-230b-4f85-89de-2438df7ad2e9",
            "durationms": 0,
            "identity": "Some Identity",
            "level": 4,
            "location": "US",
            "operationname": "Sign-in activity",
            "operationversion": 1,
            "p_event_time": "2023-07-21 05:05:12.056",
            "p_log_type": "Azure.Audit",
            "properties": {
                "appDisplayName": "Office 365 Exchange Online",
                "appId": "975c4feb-8c6b-4ed8-82e2-f0952ecc64d4",
                "appliedConditionalAccessPolicies": [],
                "authenticationContextClassReferences": [],
                "authenticationDetails": [
                    {
                        "authenticationMethod": "Password",
                        "authenticationMethodDetail": "Password in the cloud",
                        "authenticationStepDateTime": "2023-01-11T11:11:11",
                        "authenticationStepRequirement": "",
                        "authenticationStepResultDetail": "Correct password",
                        "succeeded": True,
                    },
                ],
                "authenticationProcessingDetails": [],
                "authenticationProtocol": "rpoc",
                "authenticationRequirement": "singleFactorAuthentication",
                "authenticationRequirementPolicies": [],
                "authenticationStrengths": [],
                "autonomousSystemNumber": 1234,
                "clientAppUsed": "Authenticated SMTP",
                "clientCredentialType": "clientAssertion",
                "conditionalAccessStatus": "notApplied",
                "correlationId": "9d5f7dd3-46a7-475a-9cc3-96160551ce49",
                "createdDateTime": "2023-07-21 05:03:52.160562400",
                "crossTenantAccessType": "none",
                "deviceDetail": {"deviceId": "", "displayName": "", "operatingSystem": "MacOs"},
                "flaggedForReview": False,
                "homeTenantId": "4328f0a8-06da-4457-b0d0-2c8e115e52cc",
                "id": "d9ae0e20-c959-42a3-ba74-00f59ac9f6c6",
                "incomingTokenType": "none",
                "ipAddress": "12.12.12.12",
                "isInteractive": True,
                "isTenantRestricted": False,
                "location": {
                    "city": "Springfield",
                    "countryOrRegion": "US",
                    "geoCoordinates": {"latitude": 34.55555555555555, "longitude": -74.4444444444444},
                    "state": "Virginia",
                },
                "mfaDetail": {},
                "networkLocationDetails": [],
                "originalRequestId": "51755ad1-bc0d-4694-acb2-a21f054869ab",
                "privateLinkDetails": {},
                "processingTimeInMilliseconds": 343,
                "resourceDisplayName": "Office 365 Exchange Online",
                "resourceId": "b54c474b-4d74-4920-b0c3-7a8d2f2c4e95",
                "resourceServicePrincipalId": "d8f787f8-7f22-40d4-b057-6fc59c49ca43",
                "resourceTenantId": "fc66a38e-bcba-4fe9-b2de-c67918514cc5",
                "riskDetail": "none",
                "riskEventTypes": [],
                "riskEventTypes_v2": [],
                "riskLevelAggregated": "none",
                "riskLevelDuringSignIn": "none",
                "riskState": "none",
                "rngcStatus": 0,
                "servicePrincipalId": "",
                "sessionLifetimePolicies": [],
                "ssoExtensionVersion": "",
                "status": {"additionalDetails": "MFA requirement satisfied by claim in the token", "errorCode": 0},
                "tenantId": "237c496d-1ca2-4b13-aa0f-69e44d745a27",
                "tokenIssuerName": "",
                "tokenIssuerType": "AzureAD",
                "uniqueTokenIdentifier": "hhhhhhhhhhhhhhhhhhhhhh",
                "userAgent": "BAV2ROPC",
                "userDisplayName": "A User Display Name",
                "userId": "8105c513-d24b-4b8d-9035-f8d48854d703",
                "userPrincipalName": "homer.simpson@springfield.org",
                "userType": "Member",
            },
            "resourceid": "/tenants/ef3a1b35-97d6-474b-a9de-94d21d4ee71b/providers/Microsoft.aadiam",
            "resultsignature": "None",
            "resulttype": 0,
            "tenantid": "c5fd010a-2972-4be2-9c67-986ceca2a042",
            "time": "2023-07-21 05:05:12.056",
        },
    ),
    RuleTest(
        name="Sign-In with legacy auth, KNOWN_EXCEPTION",
        expected_result=False,
        mocks=[RuleMock(object_name="KNOWN_EXCEPTIONS", return_value='[\n  "homer.simpson@springfield.org"\n]')],
        log={
            "calleripaddress": "173.53.70.163",
            "category": "SignInLogs",
            "correlationid": "1043d312-230b-4f85-89de-2438df7ad2e9",
            "durationms": 0,
            "identity": "Some Identity",
            "level": 4,
            "location": "US",
            "operationname": "Sign-in activity",
            "operationversion": 1,
            "p_event_time": "2023-07-21 05:05:12.056",
            "p_log_type": "Azure.Audit",
            "properties": {
                "appDisplayName": "Office 365 Exchange Online",
                "appId": "975c4feb-8c6b-4ed8-82e2-f0952ecc64d4",
                "appliedConditionalAccessPolicies": [],
                "authenticationContextClassReferences": [],
                "authenticationDetails": [
                    {
                        "authenticationMethod": "Password",
                        "authenticationMethodDetail": "Password in the cloud",
                        "authenticationStepDateTime": "2023-01-11T11:11:11",
                        "authenticationStepRequirement": "",
                        "authenticationStepResultDetail": "Correct password",
                        "succeeded": True,
                    },
                ],
                "authenticationProcessingDetails": [],
                "authenticationProtocol": "rpoc",
                "authenticationRequirement": "singleFactorAuthentication",
                "authenticationRequirementPolicies": [],
                "authenticationStrengths": [],
                "autonomousSystemNumber": 1234,
                "clientAppUsed": "Authenticated SMTP",
                "clientCredentialType": "clientAssertion",
                "conditionalAccessStatus": "notApplied",
                "correlationId": "9d5f7dd3-46a7-475a-9cc3-96160551ce49",
                "createdDateTime": "2023-07-21 05:03:52.160562400",
                "crossTenantAccessType": "none",
                "deviceDetail": {"deviceId": "", "displayName": "", "operatingSystem": "MacOs"},
                "flaggedForReview": False,
                "homeTenantId": "4328f0a8-06da-4457-b0d0-2c8e115e52cc",
                "id": "d9ae0e20-c959-42a3-ba74-00f59ac9f6c6",
                "incomingTokenType": "none",
                "ipAddress": "12.12.12.12",
                "isInteractive": True,
                "isTenantRestricted": False,
                "location": {
                    "city": "Springfield",
                    "countryOrRegion": "US",
                    "geoCoordinates": {"latitude": 34.55555555555555, "longitude": -74.4444444444444},
                    "state": "Virginia",
                },
                "mfaDetail": {},
                "networkLocationDetails": [],
                "originalRequestId": "51755ad1-bc0d-4694-acb2-a21f054869ab",
                "privateLinkDetails": {},
                "processingTimeInMilliseconds": 343,
                "resourceDisplayName": "Office 365 Exchange Online",
                "resourceId": "b54c474b-4d74-4920-b0c3-7a8d2f2c4e95",
                "resourceServicePrincipalId": "d8f787f8-7f22-40d4-b057-6fc59c49ca43",
                "resourceTenantId": "fc66a38e-bcba-4fe9-b2de-c67918514cc5",
                "riskDetail": "none",
                "riskEventTypes": [],
                "riskEventTypes_v2": [],
                "riskLevelAggregated": "none",
                "riskLevelDuringSignIn": "none",
                "riskState": "none",
                "rngcStatus": 0,
                "servicePrincipalId": "",
                "sessionLifetimePolicies": [],
                "ssoExtensionVersion": "",
                "status": {"additionalDetails": "MFA requirement satisfied by claim in the token", "errorCode": 0},
                "tenantId": "237c496d-1ca2-4b13-aa0f-69e44d745a27",
                "tokenIssuerName": "",
                "tokenIssuerType": "AzureAD",
                "uniqueTokenIdentifier": "hhhhhhhhhhhhhhhhhhhhhh",
                "userAgent": "BAV2ROPC",
                "userDisplayName": "A User Display Name",
                "userId": "8105c513-d24b-4b8d-9035-f8d48854d703",
                "userPrincipalName": "homer.simpson@springfield.org",
                "userType": "Member",
            },
            "resourceid": "/tenants/ef3a1b35-97d6-474b-a9de-94d21d4ee71b/providers/Microsoft.aadiam",
            "resultsignature": "None",
            "resulttype": 0,
            "tenantid": "c5fd010a-2972-4be2-9c67-986ceca2a042",
            "time": "2023-07-21 05:05:12.056",
        },
    ),
]


@panther_managed
class AzureAuditLegacyAuth(Rule):
    id = "Azure.Audit.LegacyAuth-prototype"
    display_name = "Azure SignIn via Legacy Authentication Protocol"
    dedup_period_minutes = 10
    log_types = [LogType.AZURE_AUDIT]
    default_severity = Severity.MEDIUM
    default_description = "This detection looks for Successful Logins that have used legacy authentication protocols\n"
    default_reference = (
        "https://learn.microsoft.com/en-us/azure/active-directory/reports-monitoring/workbook-legacy-authentication"
    )
    default_runbook = "Based on Microsoft's analysis more than 97 percent of credential stuffing attacks use legacy authentication and more than 99 percent of password spray attacks use legacy authentication protocols. These attacks would stop with basic authentication disabled or blocked. see https://learn.microsoft.com/en-us/azure/active-directory/conditional-access/block-legacy-authentication\nIf you are aware of this Legacy Auth need, and need to continue using this mechanism, add the principal name to KNOWN_EXCEPTIONS. The Reference link contains additional material hosted on Microsoft.com\n"
    summary_attributes = ["properties:ServicePrincipalName", "properties:UserPrincipalName", "properties:ipAddress"]
    tests = azure_audit_legacy_auth_tests
    LEGACY_AUTH_USERAGENTS = ["BAV2ROPC", "CBAInPROD"]  # CBAInPROD is reported to be IMAP
    # Add ServicePrincipalName/UserPrincipalName to
    # KNOWN_EXCEPTIONS to prevent these Principals from Alerting
    KNOWN_EXCEPTIONS = []

    def rule(self, event):
        if not is_sign_in_event(event):
            return False
        if isinstance(self.KNOWN_EXCEPTIONS, MagicMock):
            self.KNOWN_EXCEPTIONS = json.loads(self.KNOWN_EXCEPTIONS())  # pylint: disable=not-callable
        if actor_user(event) in self.KNOWN_EXCEPTIONS:
            return False
        user_agent = deep_get(event, "properties", "userAgent", default="")
        error_code = deep_get(event, "properties", "status", "errorCode", default=0)
        return all([user_agent in self.LEGACY_AUTH_USERAGENTS, error_code == 0])

    def title(self, event):
        principal = actor_user(event)
        if principal is None:
            principal = "<NO_PRINCIPALNAME>"
        return f"AzureSignIn: Principal [{principal}] authenticated with a legacy auth protocol"

    def dedup(self, event):
        principal = actor_user(event)
        if principal is None:
            principal = "<NO_PRINCIPALNAME>"
        return principal

    def alert_context(self, event):
        a_c = azure_signin_alert_context(event)
        a_c["userAgent"] = deep_get(event, "properties", "userAgent", "<NO_USERAGENT>")
        return a_c



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

duo_user_endpoint_failure_tests: list[RuleTest] = [
    RuleTest(
        name="endpoint_is_not_in_management_system",
        expected_result=True,
        log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "endpoint_is_not_in_management_system",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    RuleTest(
        name="endpoint_failed_google_verification",
        expected_result=True,
        log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "endpoint_failed_google_verification",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    RuleTest(
        name="endpoint_is_not_trusted",
        expected_result=True,
        log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "endpoint_is_not_trusted",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    RuleTest(
        name="could_not_determine_if_endpoint_was_trusted",
        expected_result=True,
        log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "could_not_determine_if_endpoint_was_trusted",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    RuleTest(
        name="invalid_device",
        expected_result=True,
        log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "invalid_device",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
    RuleTest(
        name="good_auth",
        expected_result=False,
        log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {"key": "D12345", "name": "Slack"},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "user_approved",
            "result": "success",
            "user": {"name": "example@example.io"},
        },
    ),
    RuleTest(
        name="denied_old_creds",
        expected_result=False,
        log={
            "access_device": {"ip": "12.12.112.25", "os": "Mac OS X"},
            "auth_device": {"ip": "12.12.12.12"},
            "application": {"key": "D12345", "name": "Slack"},
            "event_type": "authentication",
            "factor": "duo_push",
            "reason": "out_of_date",
            "result": "denied",
            "user": {"name": "example@example.io"},
        },
    ),
]


@panther_managed
class DUOUserEndpointFailure(Rule):
    id = "DUO.User.Endpoint.Failure-prototype"
    display_name = "Duo User Denied For Endpoint Error"
    dedup_period_minutes = 15
    log_types = [LogType.DUO_AUTHENTICATION]
    tags = ["Duo"]
    default_severity = Severity.MEDIUM
    default_description = "A Duo user's authentication was denied due to a suspicious error on the endpoint"
    default_reference = "https://duo.com/docs/adminapi#authentication-logs"
    default_runbook = "Follow up with the endpoint owner to see status. Follow up with user to verify attempts."
    tests = duo_user_endpoint_failure_tests

    def rule(self, event):
        endpoint_reasons = [
            "endpoint_is_not_in_management_system",
            "endpoint_failed_google_verification",
            "endpoint_is_not_trusted",
            "could_not_determine_if_endpoint_was_trusted",
            "invalid_device",
        ]
        return event.get("reason", "") in endpoint_reasons

    def title(self, event):
        user = deep_get(event, "user", "name", default="Unknown")
        reason = event.get("reason", "Unknown")
        return f"Duo User [{user}] encountered suspicious endpoint issue [{reason}]"

    def alert_context(self, event):
        return {
            "factor": event.get("factor"),
            "reason": event.get("reason"),
            "user": deep_get(event, "user", "name", default=""),
            "os": deep_get(event, "access_device", "os", default=""),
            "ip_access": deep_get(event, "access_device", "ip", default=""),
            "ip_auth": deep_get(event, "auth_device", "ip", default=""),
            "application": deep_get(event, "application", "name", default=""),
        }



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import deep_get

audit_log_type_map = {
    "CREATE_USER": event_type.USER_ACCOUNT_CREATED,
    "DELETE_USER": event_type.USER_ACCOUNT_DELETED,
    "UPDATE_USER": event_type.USER_ACCOUNT_MODIFIED,
    "CREATE_USER_ROLE": event_type.USER_GROUP_CREATED,
    "DELETE_USER_ROLE": event_type.USER_GROUP_DELETED,
    "UPDATE_USER_ROLE": event_type.USER_ROLE_MODIFIED,
}


def get_event_type(event):
    audit_log_type = event.get("actionName")
    matched = audit_log_type_map.get(audit_log_type)
    if matched is not None:
        return matched
    return None


def get_actor_user(event):
    # First prefer actor.attributes.email
    #  automatons like SCIM won't have an actor.attributes.email
    actor_user = deep_get(event, "actor", "attributes", "email")
    if actor_user is None:
        actor_user = deep_get(event, "actor", "id")
    return actor_user


class StandardPantherAudit(DataModel):
    id: str = "Standard.Panther.Audit"
    display_name: str = "Panther Audit Logs"
    enabled: bool = True
    log_types: list[str] = [LogType.PANTHER_AUDIT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="source_ip", path="sourceIP"),
        DataModelMapping(name="user_agent", path="userAgent"),
        DataModelMapping(name="actor_user", method=get_actor_user),
        DataModelMapping(name="user", path="$.actionParams.input.email"),
        DataModelMapping(name="event_type", method=get_event_type),
    ]



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

retrieve_ss_oaccesstoken_tests: list[RuleTest] = [
    RuleTest(
        name="Retrieve SSO access token",
        expected_result=True,
        log={
            "eventName": "CreateToken",
            "eventSource": "sso.amazonaws.com",
            "eventVersion": "1.08",
            "recipientAccountId": "<organization master account ID>",
            "requestParameters": {
                "clientId": "...",
                "clientSecret": "HIDDEN_DUE_TO_SECURITY_REASONS",
                "deviceCode": "...",
                "grantType": "urn:ietf:params:oauth:grant-type:device_code",
            },
            "responseElements": {
                "accessToken": "HIDDEN_DUE_TO_SECURITY_REASONS",
                "expiresIn": 28800,
                "idToken": "HIDDEN_DUE_TO_SECURITY_REASONS",
                "refreshToken": "HIDDEN_DUE_TO_SECURITY_REASONS",
                "tokenType": "Bearer",
            },
            "sourceIPAddress": "<Attacker source IP>",
            "userAgent": "<Attacker user agent (here: Boto3/1.17.80 Python/3.9.5 Darwin/20.3.0 Botocore/1.20.80)>",
            "userIdentity": {
                "accountId": "<organization master account ID>",
                "principalId": "<internal victim user id>",
                "type": "Unknown",
                "userName": "<victim display name>",
            },
        },
    ),
]


@panther_managed
class RetrieveSSOaccesstoken(Rule):
    id = "Retrieve.SSO.access.token-prototype"
    display_name = "SIGNAL - Retrieve SSO access token"
    create_alert = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    default_severity = Severity.INFO
    tests = retrieve_ss_oaccesstoken_tests

    def rule(self, event):
        return event.get("eventSource") == "sso.amazonaws.com" and event.get("eventName") == "CreateToken"



import inspect
import json
from typing import Any, Type

from prettytable import PrettyTable

from pypanther import utils
from pypanther.base import Rule

DEFAULT_RULE_TABLE_ATTRS = [
    "id",
    "log_types",
    "default_severity",
    "enabled",
]

VALID_RULE_TABLE_ATTRS = [
    *DEFAULT_RULE_TABLE_ATTRS,
    "create_alert",
    "dedup_period_minutes",
    "display_name",
    "summary_attributes",
    "threshold",
    "tags",
    "default_description",
    "default_reference",
    "default_runbook",
    "default_destinations",
]

OUTPUT_TYPE_JSON = "json"
OUTPUT_TYPE_TEXT = "text"
DEFAULT_CLI_OUTPUT_TYPE = OUTPUT_TYPE_TEXT
VALID_CLI_OUTPUT_TYPES = [
    OUTPUT_TYPE_TEXT,
    OUTPUT_TYPE_JSON,
]

JSON_INDENT_LEVEL = 2


def print_rule_table(rules: list[Type[Rule]], attributes: list[str] | None = None) -> None:
    """
    Prints rules in a table format for easy viewing.

    Parameters
    ----------
        rules (list[Type[Rule]]): The list of PantherRule subclasses that will be printed in table format.
        attributes (list[str] | None): The list of attributes that will appear as columns in the table.
            Supplying None or an empty list will use defaults of [id, log_types, default_severity, enabled].

    """
    attributes = utils.dedup_list_preserving_order(attributes or [])
    check_rule_attributes(attributes)

    if len(attributes) == 0:
        attributes = DEFAULT_RULE_TABLE_ATTRS

    table = PrettyTable()
    table.field_names = attributes

    for rule in rules:
        table.add_row([getattr(rule, attr) if attr != "log_types" else fmt_log_types_attr(rule) for attr in attributes])

    # sort the table by the first attr given or the ID
    # sortby must be set before setting sort_key
    table.sortby = "id" if "id" in attributes else (attributes or [])[0]

    # sort all columns in alphanumeric order by joining them
    def key(row: list[Any]) -> list[Any]:
        # row[0] is the sortby attr, row[1:] are all attrs in the row
        # for example: [id, id, log_type, enabled]
        # by replacing the [0] item we replace what it sorts by
        row[0] = "".join(str(val) for val in row[1:])
        return row

    table.sort_key = key

    print(table)


def fmt_log_types_attr(rule: Type[Rule]) -> str:
    log_types = rule.log_types
    if len(log_types) > 2:
        log_types = log_types[:2] + [f"+{len(log_types) - 2}"]

    return ", ".join([str(s) for s in log_types])


def print_rules_as_json(rules: list[Type[Rule]], attributes: list[str] | None = None) -> None:
    """
    Prints rules in JSON format for easy viewing.

    Parameters
    ----------
        rules (list[Type[Rule]]): The list of PantherRule subclasses that will be printed in JSON format.
        attributes (list[str] | None): The list of attributes that will appear as attributes in the JSON.
            Supplying None or an empty list will use defaults of [id, log_types, default_severity, enabled].

    """
    attributes = utils.dedup_list_preserving_order(attributes or [])
    check_rule_attributes(attributes)

    if len(attributes) == 0:
        attributes = DEFAULT_RULE_TABLE_ATTRS

    rule_dicts = [{attr: getattr(rule, attr) for attr in attributes} for rule in rules]
    print(json.dumps(rule_dicts, indent=JSON_INDENT_LEVEL))


def check_rule_attributes(attributes: list[str]) -> None:
    for attr in attributes or []:
        if attr not in VALID_RULE_TABLE_ATTRS:
            raise AttributeError(f"Attribute '{attr}' is not allowed.")


def print_rule_as_json(rule: Type[Rule]) -> None:
    source = inspect.getsource(rule)
    rule_dict = rule.asdict()
    del rule_dict["tests"]
    rule_dict["class_definition"] = source
    rule_json = json.dumps(rule_dict, indent=JSON_INDENT_LEVEL)
    print(rule_json)


def print_rule_as_text(rule: Type[Rule]) -> None:
    print(inspect.getsource(rule))



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

awsec2_monitoring_tests: list[RuleTest] = [
    RuleTest(
        name="CopyImage",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "0ea3f05a-066c-43f9-8869-393ba67e7936",
            "eventName": "CreateImage",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-09-29 22:25:17",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123456789101"],
            "p_any_aws_arns": [
                "arn:aws:iam::123456789101:role/DevAdministrator",
                "arn:aws:sts::123456789101:assumed-role/DevAdministrator/test_user",
            ],
            "p_any_aws_instance_ids": ["i-0381a3817f72a949d"],
            "p_any_domain_names": ["AWS Internal"],
            "p_any_trace_ids": ["ASIA5PZQZ5QHE2FUNXHR"],
            "p_event_time": "2022-09-29 22:25:17",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-09-29 22:27:25.748",
            "p_row_id": "66011977ec1fd0cf9dacf7d913f08d06",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "CloudTrail Logs",
            "readOnly": False,
            "recipientAccountId": "123456789101",
            "requestID": "e686939a-a08a-4fd6-abf5-9ea34793cf25",
            "requestParameters": {
                "blockDeviceMapping": {
                    "items": [{"deviceName": "/dev/xvda", "ebs": {"deleteOnTermination": True, "volumeSize": 8}}],
                },
                "instanceId": "i-0381a3817f72a949d",
                "name": "testimage",
                "noReboot": False,
            },
            "responseElements": {
                "imageId": "ami-06aaf5e4b77161786",
                "requestId": "e686939a-a08a-4fd6-abf5-9ea34793cf25",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA5PZQZ5QHE2FUNXHR",
                "accountId": "123456789101",
                "arn": "arn:aws:sts::123456789101:assumed-role/DevAdministrator/test_user",
                "principalId": "AROA5PZQZ5QHBULW27VAC:test_user",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-29T22:22:46Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789101",
                        "arn": "arn:aws:iam::123456789101:role/DevAdministrator",
                        "principalId": "AROA5PZQZ5QHBULW27VAC",
                        "type": "Role",
                        "userName": "DevAdministrator",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="RunInstance",
        expected_result=False,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "015c585b-cbc2-4f9e-9c52-a2f22f3c09f4",
            "eventName": "RunInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-10-20 14:16:43",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123123123123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123123123123:role/DevAdministrator",
                "arn:aws:sts::123123123123:assumed-role/DevAdministrator/temp_user",
            ],
            "p_any_aws_instance_ids": ["i-0d7e4b9be8a0de6ea"],
            "p_any_aws_tags": ["Name:test2"],
            "p_any_domain_names": ["AWS Internal", "ip-111.111.111.111.ec2.internal"],
            "p_any_ip_addresses": ["111.111.111.111"],
            "p_any_trace_ids": ["ASIA5PZQZ5QHAW6RFPO5"],
            "p_any_usernames": ["DevAdministrator"],
            "p_event_time": "2022-10-20 14:16:43",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-10-20 14:21:06.845",
            "p_row_id": "be7b3bed9bb891e4a8b9f38d149909",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123123123123",
            "requestID": "25c95576-a825-44fd-971d-5a52c1e3b2be",
            "requestParameters": {
                "blockDeviceMapping": {},
                "disableApiStop": False,
                "disableApiTermination": False,
                "ebsOptimized": False,
                "instanceType": "t1.micro",
                "instancesSet": {
                    "items": [{"imageId": "ami-026b57f3c383c2eec", "keyName": "kp1", "maxCount": 1, "minCount": 1}],
                },
                "monitoring": {"enabled": False},
                "networkInterfaceSet": {
                    "items": [
                        {
                            "associatePublicIpAddress": True,
                            "deviceIndex": 0,
                            "groupSet": {"items": [{"groupId": "sg-0aebfa21f302bded9"}]},
                        },
                    ],
                },
                "privateDnsNameOptions": {
                    "enableResourceNameDnsAAAARecord": False,
                    "enableResourceNameDnsARecord": True,
                    "hostnameType": "ip-name",
                },
                "tagSpecificationSet": {
                    "items": [{"resourceType": "instance", "tags": [{"key": "Name", "value": "test2"}]}],
                },
            },
            "responseElements": {
                "groupSet": {},
                "instancesSet": {
                    "items": [
                        {
                            "amiLaunchIndex": 0,
                            "architecture": "x86_64",
                            "blockDeviceMapping": {},
                            "capacityReservationSpecification": {"capacityReservationPreference": "open"},
                            "cpuOptions": {"coreCount": 1, "threadsPerCore": 1},
                            "currentInstanceBootMode": "bios",
                            "ebsOptimized": False,
                            "enaSupport": True,
                            "enclaveOptions": {"enabled": False},
                            "groupSet": {
                                "items": [{"groupId": "sg-0aebfa21f302bded9", "groupName": "launch-wizard-4"}],
                            },
                            "hypervisor": "xen",
                            "imageId": "ami-026b57f3c383c2eec",
                            "instanceId": "i-0d7e4b9be8a0de6ea",
                            "instanceState": {"code": 0, "name": "pending"},
                            "instanceType": "t1.micro",
                            "keyName": "kp1",
                            "launchTime": 1666275403000.0,
                            "maintenanceOptions": {"autoRecovery": "default"},
                            "metadataOptions": {
                                "httpEndpoint": "enabled",
                                "httpProtocolIpv4": "enabled",
                                "httpProtocolIpv6": "disabled",
                                "httpPutResponseHopLimit": 1,
                                "httpTokens": "optional",
                                "instanceMetadataTags": "disabled",
                                "state": "pending",
                            },
                            "monitoring": {"state": "disabled"},
                            "networkInterfaceSet": {
                                "items": [
                                    {
                                        "attachment": {
                                            "attachTime": 1666275403000.0,
                                            "attachmentId": "eni-attach-0f01abecb268392c1",
                                            "deleteOnTermination": True,
                                            "deviceIndex": 0,
                                            "networkCardIndex": 0,
                                            "status": "attaching",
                                        },
                                        "groupSet": {
                                            "items": [
                                                {"groupId": "sg-0aebfa21f302bded9", "groupName": "launch-wizard-4"},
                                            ],
                                        },
                                        "interfaceType": "interface",
                                        "ipv6AddressesSet": {},
                                        "macAddress": "0a:47:b4:21:fe:8d",
                                        "networkInterfaceId": "eni-08b298299ee7c922c",
                                        "ownerId": "123123123123",
                                        "privateDnsName": "ip-111.111.111.111.ec2.internal",
                                        "privateIpAddress": "111.111.111.111",
                                        "privateIpAddressesSet": {
                                            "item": [
                                                {
                                                    "primary": True,
                                                    "privateDnsName": "ip-111.111.111.111.ec2.internal",
                                                    "privateIpAddress": "111.111.111.111",
                                                },
                                            ],
                                        },
                                        "sourceDestCheck": True,
                                        "status": "in-use",
                                        "subnetId": "subnet-0ae6d533cb0b18193",
                                        "tagSet": {},
                                        "vpcId": "vpc-0f59e8f1222b0de6a",
                                    },
                                ],
                            },
                            "placement": {"availabilityZone": "us-east-1a", "tenancy": "default"},
                            "privateDnsName": "ip-111.111.111.111.ec2.internal",
                            "privateDnsNameOptions": {
                                "enableResourceNameDnsAAAARecord": False,
                                "enableResourceNameDnsARecord": True,
                                "hostnameType": "ip-name",
                            },
                            "privateIpAddress": "111.111.111.111",
                            "productCodes": {},
                            "rootDeviceName": "/dev/xvda",
                            "rootDeviceType": "ebs",
                            "sourceDestCheck": True,
                            "stateReason": {"code": "pending", "message": "pending"},
                            "subnetId": "subnet-0ae6d533cb0b18193",
                            "tagSet": {"items": [{"key": "Name", "value": "test2"}]},
                            "virtualizationType": "hvm",
                            "vpcId": "vpc-0f59e8f1222b0de6a",
                        },
                    ],
                },
                "ownerId": "123123123123",
                "requestId": "25c95576-a825-44fd-971d-5a52c1e3b2be",
                "reservationId": "r-02debcf2c4878bc7f",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA5PZQZ5QHAW6RFPO5",
                "accountId": "123123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/DevAdministrator/temp_user",
                "principalId": "AROA5PZQZ5QHBULW27VAC:temp_user",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-10-20T14:14:22Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/DevAdministrator",
                        "principalId": "AROA5PZQZ5QHBULW27VAC",
                        "type": "Role",
                        "userName": "DevAdministrator",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="RunInstance - Dry Run ",
        expected_result=False,
        log={
            "awsRegion": "us-west-2",
            "errorCode": "Client.DryRunOperation",
            "errorMessage": "Request would have succeeded, but DryRun flag is set.",
            "eventCategory": "Management",
            "eventID": "ab804e72-7237-49c6-8f20-c3ef09859e78",
            "eventName": "RunInstances",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-10-13 16:35:33",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_any_aws_account_ids": ["123123123123"],
            "p_any_aws_arns": [
                "arn:aws:iam::123123123123:role/aws-service-role/eks-nodegroup.amazonaws.com/AWSServiceRoleForAmazonEKSNodegroup",
                "arn:aws:sts::123123123123:assumed-role/AWSServiceRoleForAmazonEKSNodegroup/EKS",
            ],
            "p_any_domain_names": ["eks-nodegroup.amazonaws.com"],
            "p_any_usernames": ["AWSServiceRoleForAmazonEKSNodegroup"],
            "p_event_time": "2022-10-13 16:35:33",
            "p_log_type": "AWS.CloudTrail",
            "p_parse_time": "2022-10-13 16:42:16.583",
            "p_row_id": "464736612772b0bd8dd8c5fc1384d209",
            "p_source_id": "125a8146-e3ea-454b-aed7-9e08e735b670",
            "p_source_label": "Panther Identity Org CloudTrail",
            "readOnly": False,
            "recipientAccountId": "123123123123",
            "requestID": "5e7636b9-d5b5-4bb1-8e37-f98ae7c04e52",
            "requestParameters": {
                "availabilityZone": "us-west-2b",
                "blockDeviceMapping": {},
                "clientToken": "27bec563-7673-4b80-8e00-3537e3b7ad6b",
                "disableApiStop": False,
                "disableApiTermination": False,
                "instanceType": "m5.xlarge",
                "instancesSet": {"items": [{"maxCount": 1, "minCount": 1}]},
                "launchTemplate": {"launchTemplateId": "lt-0622a7ff26539376a", "version": "4"},
                "monitoring": {"enabled": False},
                "subnetId": "subnet-0d465e7cad854a993",
            },
            "sourceIPAddress": "eks-nodegroup.amazonaws.com",
            "userAgent": "eks-nodegroup.amazonaws.com",
            "userIdentity": {
                "accountId": "123123123123",
                "arn": "arn:aws:sts::123123123123:assumed-role/AWSServiceRoleForAmazonEKSNodegroup/EKS",
                "invokedBy": "eks-nodegroup.amazonaws.com",
                "principalId": "AROAZBD2CNPWD5DHZ366F:EKS",
                "sessionContext": {
                    "attributes": {"creationDate": "2022-10-13T16:35:31Z", "mfaAuthenticated": "false"},
                    "sessionIssuer": {
                        "accountId": "123123123123",
                        "arn": "arn:aws:iam::123123123123:role/aws-service-role/eks-nodegroup.amazonaws.com/AWSServiceRoleForAmazonEKSNodegroup",
                        "principalId": "AROAZBD2CNPWD5DHZ366F",
                        "type": "Role",
                        "userName": "AWSServiceRoleForAmazonEKSNodegroup",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
    RuleTest(
        name="CopyImage - UserIdentity Null",
        expected_result=True,
        log={
            "awsRegion": "us-east-1",
            "eventCategory": "Management",
            "eventID": "0ea3f05a-066c-43f9-8869-393ba67e7936",
            "eventName": "CreateImage",
            "eventSource": "ec2.amazonaws.com",
            "eventTime": "2022-09-29 22:25:17",
            "eventType": "AwsApiCall",
            "eventVersion": "1.08",
            "managementEvent": True,
            "readOnly": False,
            "recipientAccountId": "123456789101",
            "requestID": "e686939a-a08a-4fd6-abf5-ffffffffffff",
            "requestParameters": {
                "blockDeviceMapping": {
                    "items": [{"deviceName": "/dev/xvda", "ebs": {"deleteOnTermination": True, "volumeSize": 8}}],
                },
                "instanceId": "i-0381a3817f72a949d",
                "name": "testimage",
                "noReboot": False,
            },
            "responseElements": {
                "imageId": "ami-06aaf5e4b77161786",
                "requestId": "e686939a-a08a-4fd6-abf5-9ea34793cf25",
            },
            "sessionCredentialFromConsole": True,
            "sourceIPAddress": "AWS Internal",
            "userAgent": "AWS Internal",
            "userIdentity": {
                "accessKeyId": "ASIA5PZQZ5QHE2FUNXHR",
                "accountId": "123456789101",
                "arn": "arn:aws:sts::123456789101:assumed-role/DevAdministrator/test_user",
                "principalId": "AROA5PZQZ5QHBULW27VAC:test_user",
                "invokedBy": None,
                "sessionContext": {
                    "attributes": {"creationDate": "2022-09-29T22:22:46Z", "mfaAuthenticated": "true"},
                    "sessionIssuer": {
                        "accountId": "123456789101",
                        "arn": "arn:aws:iam::123456789101:role/DevAdministrator",
                        "principalId": "AROA5PZQZ5QHBULW27VAC",
                        "type": "Role",
                        "userName": "DevAdministrator",
                    },
                    "webIdFederationData": {},
                },
                "type": "AssumedRole",
            },
        },
    ),
]


@panther_managed
class AWSEC2Monitoring(Rule):
    default_description = "Checks CloudTrail for occurrences of EC2 Image Actions."
    display_name = "AWS EC2 Image Monitoring"
    reports = {"MITRE ATT&CK": ["TA0002:T1204"]}
    default_runbook = "Verify that the action was not taken by a malicious actor."
    default_reference = "https://docs.aws.amazon.com/service-authorization/latest/reference/list_amazonec2imagebuilder.html#amazonec2imagebuilder-actions-as-permissions"
    default_severity = Severity.INFO
    tags = ["ec2"]
    log_types = [LogType.AWS_CLOUDTRAIL]
    id = "AWS.EC2.Monitoring-prototype"
    tests = awsec2_monitoring_tests
    # AWS CloudTrail API eventNames for EC2 Image Actions
    EC2_IMAGE_ACTIONS = [
        "CopyFpgaImage",
        "CopyImage",
        "CreateFpgaImage",
        "CreateImage",
        "CreateRestoreImageTask",
        "CreateStoreImageTask",
        "ImportImage",
    ]

    def rule(self, event):
        # Disqualify any eventSource that is not ec2
        if event.get("eventSource", "") != "ec2.amazonaws.com":
            return False
        # Disqualify AWS Service-Service operations, which can appear in a variety of forms
        if (
            event.get("sourceIPAddress", "").endswith(".amazonaws.com")
            or deep_get(event, "userIdentity", "type", default="") == "AWSService"
            or deep_get(event, "userIdentity", "invokedBy", default="") == "AWS Internal"
            or deep_get(event, "userIdentity", "invokedBy", default="").endswith(".amazonaws.com")
        ):
            # FYI there is a weird quirk in the sourceIPAddress field of CloudTrail
            #  events with ec2.amazonaws.com as the source name where users of the
            #  web-console will have their sourceIPAddress recorded as "AWS Internal"
            #  though their userIdentity will be more normal.
            #  Example cloudtrail event in the "Terminate instance From WebUI with assumedRole" test
            return False
        # Dry run operations get logged as SES Internal in the sourceIPAddress
        #  but not in the invokedBy field
        if event.get("errorCode", "") == "Client.DryRunOperation":
            return False
        # Disqualify any eventNames that do not Include Image Actions
        # and events that have readOnly set to false
        if event.get("eventName", "") in self.EC2_IMAGE_ACTIONS:
            return True
        return False

    def title(self, event):
        return f"[{deep_get(event, 'userIdentity', 'sessionContext', 'sessionIssuer', 'userName')}] triggered a CloudTrail action [{event.get('eventName')}] within AWS Account ID: [{event.get('recipientAccountId')}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get
from pypanther.helpers.panther_base_helpers import gsuite_details_lookup as details_lookup
from pypanther.helpers.panther_base_helpers import gsuite_parameter_lookup as param_lookup

g_suite_drive_overly_visible_tests: list[RuleTest] = [
    RuleTest(
        name="Access Event",
        expected_result=False,
        log={
            "p_row_id": "111222",
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "drive"},
            "events": [{"type": "access", "name": "download"}],
        },
    ),
    RuleTest(
        name="Modify Event Without Over Visibility",
        expected_result=False,
        log={
            "p_row_id": "111222",
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "drive"},
            "events": [{"type": "access", "name": "edit", "parameters": [{"name": "visibility", "value": "private"}]}],
        },
    ),
    RuleTest(
        name="Overly Visible Doc Modified",
        expected_result=True,
        log={
            "p_row_id": "111222",
            "actor": {"email": "bobert@example.com"},
            "id": {"applicationName": "drive"},
            "events": [
                {
                    "type": "access",
                    "name": "edit",
                    "parameters": [
                        {"name": "visibility", "value": "people_with_link"},
                        {"name": "doc_title", "value": "my shared document"},
                    ],
                },
            ],
        },
    ),
    RuleTest(
        name="Overly Visible Doc Modified - no email",
        expected_result=True,
        log={
            "p_row_id": "111222",
            "actor": {"profileId": "1234567890123"},
            "id": {"applicationName": "drive"},
            "events": [
                {
                    "type": "access",
                    "name": "edit",
                    "parameters": [
                        {"name": "visibility", "value": "people_with_link"},
                        {"name": "doc_title", "value": "my shared document"},
                    ],
                },
            ],
        },
    ),
]


@panther_managed
class GSuiteDriveOverlyVisible(Rule):
    id = "GSuite.DriveOverlyVisible-prototype"
    display_name = "GSuite Overly Visible Drive Document"
    log_types = [LogType.GSUITE_REPORTS]
    tags = ["GSuite", "Collection:Data from Information Repositories"]
    reports = {"MITRE ATT&CK": ["TA0009:T1213"]}
    default_severity = Severity.INFO
    default_description = "A Google drive resource that is overly visible has been modified.\n"
    default_reference = (
        "https://support.google.com/docs/answer/2494822?hl=en&co=GENIE.Platform%3DDesktop&sjid=864417124752637253-EU"
    )
    default_runbook = "Investigate whether the drive document is appropriate to be this visible.\n"
    summary_attributes = ["actor:email"]
    dedup_period_minutes = 360
    tests = g_suite_drive_overly_visible_tests
    RESOURCE_CHANGE_EVENTS = {"create", "move", "upload", "edit"}
    PERMISSIVE_VISIBILITY = {"people_with_link", "public_on_the_web"}

    def rule(self, event):
        if deep_get(event, "id", "applicationName") != "drive":
            return False
        details = details_lookup("access", self.RESOURCE_CHANGE_EVENTS, event)
        return bool(details) and param_lookup(details.get("parameters", {}), "visibility") in self.PERMISSIVE_VISIBILITY

    def dedup(self, event):
        user = deep_get(event, "actor", "email")
        if user is None:
            user = deep_get(event, "actor", "profileId", default="<UNKNOWN_PROFILEID>")
        return user

    def title(self, event):
        details = details_lookup("access", self.RESOURCE_CHANGE_EVENTS, event)
        doc_title = param_lookup(details.get("parameters", {}), "doc_title")
        share_settings = param_lookup(details.get("parameters", {}), "visibility")
        user = deep_get(event, "actor", "email")
        if user is None:
            user = deep_get(event, "actor", "profileId", default="<UNKNOWN_PROFILEID>")
        return f"User [{user}] modified a document [{doc_title}] that has overly permissive share settings [{share_settings}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import m365_alert_context

microsoft365_brute_force_loginby_user_tests: list[RuleTest] = [
    RuleTest(
        name="Failed Login event",
        expected_result=True,
        log={
            "Actor": [
                {"ID": "012345-abcde-543-xyz", "Type": 0},
                {"ID": "sample.user@yourorg.onmicrosoft.com", "Type": 5},
            ],
            "ActorContextId": "123-abc-xyz-567",
            "ActorIpAddress": "1.2.3.4",
            "ApplicationId": "123-abc-sfa-321",
            "AzureActiveDirectoryEventType": 1,
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 15:57:57",
            "ExtendedProperties": [
                {"Name": "ResultStatusDetail", "Value": "Success"},
                {
                    "Name": "UserAgent",
                    "Value": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
                },
                {"Name": "UserAuthenticationMethod", "Value": "1"},
                {"Name": "RequestType", "Value": "Login:login"},
            ],
            "Id": "abc-def-123",
            "InterSystemsId": "987-432-123",
            "IntraSystemId": "aaa-bbb-ccc",
            "LogonError": "InvalidUserNameOrPassword",
            "ObjectId": "aa-11-22-bb",
            "Operation": "UserLoginFailed",
            "OrganizationId": "11-aa-22-bb",
            "RecordType": 15,
            "ResultStatus": "Success",
            "SupportTicketId": "",
            "Target": [{"ID": "11-22-33", "Type": 0}],
            "TargetContextId": "11-22-33-44",
            "UserId": "sample.user@yourorg.onmicrosoft.com",
            "UserKey": "012345-abcde-543-xyz",
            "UserType": 0,
            "Workload": "AzureActiveDirectory",
        },
    ),
    RuleTest(
        name="Login Event",
        expected_result=False,
        log={
            "Actor": [
                {"ID": "012345-abcde-543-xyz", "Type": 0},
                {"ID": "sample.user@yourorg.onmicrosoft.com", "Type": 5},
            ],
            "ActorContextId": "123-abc-xyz-567",
            "ActorIpAddress": "1.2.3.4",
            "ApplicationId": "123-abc-sfa-321",
            "AzureActiveDirectoryEventType": 1,
            "ClientIP": "1.2.3.4",
            "CreationTime": "2022-12-12 15:57:57",
            "ExtendedProperties": [
                {"Name": "ResultStatusDetail", "Value": "Success"},
                {
                    "Name": "UserAgent",
                    "Value": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/107.0.0.0 Safari/537.36",
                },
                {"Name": "RequestType", "Value": "Login:reprocess"},
            ],
            "Id": "abc-def-123",
            "InterSystemsId": "987-432-123",
            "IntraSystemId": "aaa-bbb-ccc",
            "ObjectId": "aa-11-22-bb",
            "Operation": "UserLoggedIn",
            "OrganizationId": "11-aa-22-bb",
            "RecordType": 15,
            "ResultStatus": "Success",
            "SupportTicketId": "",
            "Target": [{"ID": "11-22-33", "Type": 0}],
            "TargetContextId": "11-22-33-44",
            "UserId": "sample.user@yourorg.onmicrosoft.com",
            "UserKey": "012345-abcde-543-xyz",
            "UserType": 0,
        },
    ),
]


@panther_managed
class Microsoft365BruteForceLoginbyUser(Rule):
    default_description = "A Microsoft365 user was denied login access several times"
    display_name = "Microsoft365 Brute Force Login by User"
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_runbook = "Analyze the IP they came from and actions taken before/after."
    default_reference = "https://learn.microsoft.com/en-us/microsoft-365/troubleshoot/authentication/access-denied-when-connect-to-office-365"
    default_severity = Severity.MEDIUM
    log_types = [LogType.MICROSOFT365_AUDIT_AZURE_ACTIVE_DIRECTORY]
    id = "Microsoft365.Brute.Force.Login.by.User-prototype"
    threshold = 10
    tests = microsoft365_brute_force_loginby_user_tests

    def rule(self, event):
        return event.get("Operation", "") == "UserLoginFailed"

    def title(self, event):
        return f"Microsoft365: [{event.get('UserId', '<user-not-found>')}] may be undergoing a Brute Force Attack."

    def alert_context(self, event):
        return m365_alert_context(event)



import argparse
from typing import Any, Dict

from dynaconf import Dynaconf, Validator

from pypanther.vendor.panther_analysis_tool.constants import CONFIG_FILE


def setup_dynaconf() -> Dict[str, Any]:
    config_file_settings_raw = Dynaconf(
        settings_file=CONFIG_FILE,
        envvar_prefix="PANTHER",
        validators=[
            Validator("API_TOKEN", is_type_of=str),
            Validator("API_HOST", is_type_of=str),
        ],
    )
    # Dynaconf stores its keys in ALL CAPS
    return {k.lower(): v for k, v in config_file_settings_raw.as_dict().items()}


def dynaconf_argparse_merge(argparse_dict: Dict[str, Any], config_file_settings: Dict[str, Any]) -> None:
    # Set up another parser w/ no defaults
    aux_parser = argparse.ArgumentParser(argument_default=argparse.SUPPRESS)
    for k in argparse_dict:
        arg_name = k.replace("_", "-")
        if isinstance(argparse_dict[k], bool):
            aux_parser.add_argument("--" + arg_name, action="store_true")
        else:
            aux_parser.add_argument("--" + arg_name)
    # cli_args only contains args that were passed in the command line
    cli_args, _ = aux_parser.parse_known_args()
    for key, value in config_file_settings.items():
        if key not in cli_args:
            argparse_dict[key] = value



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_mongodb_helpers import mongodb_alert_context

mongo_db_alerting_disabled_or_deleted_tests: list[RuleTest] = [
    RuleTest(
        name="Alert added",
        expected_result=False,
        log={
            "alertConfigId": "alert_id",
            "created": "2024-04-01 11:57:54.000000000",
            "currentValue": {},
            "eventTypeName": "ALERT_CONFIG_ADDED_AUDIT",
            "id": "alert_id",
            "isGlobalAdmin": False,
            "links": [],
            "orgId": "some_org_id",
            "remoteAddress": "1.2.3.4",
            "userId": "user_id",
            "username": "some_user@company.com",
        },
    ),
    RuleTest(
        name="Alert deleted",
        expected_result=True,
        log={
            "alertConfigId": "alert_id",
            "created": "2024-04-01 11:58:52.000000000",
            "currentValue": {},
            "eventTypeName": "ALERT_CONFIG_DELETED_AUDIT",
            "id": "alert_id",
            "isGlobalAdmin": False,
            "links": [],
            "orgId": "some_org_id",
            "remoteAddress": "1.2.3.4",
            "userId": "user_id",
            "username": "some_user@company.com",
        },
    ),
]


@panther_managed
class MongoDBAlertingDisabledOrDeleted(Rule):
    default_description = "MongoDB provides security alerting policies for notifying admins when certain conditions are met. This rule detects when these policies are disabled or deleted."
    display_name = "MongoDB security alerts disabled or deleted"
    log_types = [LogType.MONGODB_ORGANIZATION_EVENT]
    id = "MongoDB.Alerting.Disabled.Or.Deleted-prototype"
    default_severity = Severity.HIGH
    reports = {"MITRE ATT&CK": ["TA0005:T1562.001"]}
    default_reference = "https://www.mongodb.com/docs/atlas/configure-alerts/"
    default_runbook = "Re-enable security alerts"
    tests = mongo_db_alerting_disabled_or_deleted_tests

    def rule(self, event):
        return event.deep_get("eventTypeName", default="") in [
            "ALERT_CONFIG_DISABLED_AUDIT",
            "ALERT_CONFIG_DELETED_AUDIT",
        ]

    def title(self, event):
        user = event.deep_get("username", default="<USER_NOT_FOUND>")
        alert_id = event.deep_get("alertConfigId", default="<ALERT_NOT_FOUND>")
        return f"MongoDB: [{user}] has disabled or deleted security alert [{alert_id}]"

    def alert_context(self, event):
        context = mongodb_alert_context(event)
        context["alertConfigId"] = event.deep_get("alertConfigId", default="<ALERT_NOT_FOUND>")
        return context



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

osquery_mac_auto_update_enabled_tests: list[RuleTest] = [
    RuleTest(
        name="Auto Updates Disabled",
        expected_result=True,
        log={
            "columns": {"domain": "com.apple.SoftwareUpdate", "key": "AutomaticCheckEnabled", "value": "false"},
            "action": "added",
            "name": "pack/mac-cis/SoftwareUpdate",
        },
    ),
    RuleTest(
        name="Auto Updates Enabled",
        expected_result=False,
        log={
            "columns": {"domain": "com.apple.SoftwareUpdate", "key": "AutomaticCheckEnabled", "value": "true"},
            "action": "added",
            "name": "pack/mac-cis/SoftwareUpdate",
        },
    ),
    RuleTest(
        name="Wrong Key",
        expected_result=False,
        log={
            "columns": {"domain": "com.apple.SoftwareUpdate", "key": "LastFullSuccessfulDate", "value": "false"},
            "action": "added",
            "name": "pack/mac-cis/SoftwareUpdate",
        },
    ),
]


@panther_managed
class OsqueryMacAutoUpdateEnabled(Rule):
    id = "Osquery.Mac.AutoUpdateEnabled-prototype"
    display_name = "OSQuery Reports Application Firewall Disabled"
    log_types = [LogType.OSQUERY_DIFFERENTIAL]
    tags = ["Osquery", "MacOS", "Security Control", "Defense Evasion:Impair Defenses"]
    reports = {"CIS": ["1.2"], "MITRE ATT&CK": ["TA0005:T1562"]}
    default_severity = Severity.MEDIUM
    dedup_period_minutes = 1440
    default_description = "Verifies that MacOS has automatic software updates enabled.\n"
    default_runbook = "Enable the auto updates on the host.\n"
    default_reference = "https://support.apple.com/en-gb/guide/mac-help/mchlpx1065/mac"
    summary_attributes = ["name", "action", "p_any_ip_addresses", "p_any_domain_names"]
    tests = osquery_mac_auto_update_enabled_tests

    def rule(self, event):
        # Send an alert if not set to "true"
        return (
            "SoftwareUpdate" in event.get("name", [])
            and event.get("action") == "added"
            and (deep_get(event, "columns", "domain") == "com.apple.SoftwareUpdate")
            and (deep_get(event, "columns", "key") == "AutomaticCheckEnabled")
            and (deep_get(event, "columns", "value") == "false")
        )



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, slack_alert_context

slack_audit_logs_app_access_expanded_tests: list[RuleTest] = [
    RuleTest(
        name="App Scopes Expanded",
        expected_result=True,
        log={
            "action": "app_scopes_expanded",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 16:48:14",
            "details": {
                "granular_bot_token": True,
                "is_internal_integration": False,
                "is_token_rotation_enabled_app": False,
                "new_scopes": [
                    "app_mentions:read",
                    "channels:join",
                    "channels:read",
                    "chat:write",
                    "chat:write.public",
                    "team:read",
                    "users:read",
                    "im:history",
                    "groups:read",
                    "reactions:write",
                    "groups:history",
                    "channels:history",
                ],
                "previous_scopes": [
                    "app_mentions:read",
                    "commands",
                    "channels:join",
                    "channels:read",
                    "chat:write",
                    "chat:write.public",
                    "users:read",
                    "groups:read",
                    "reactions:write",
                    "groups:history",
                    "channels:history",
                ],
            },
            "entity": {
                "type": "workspace",
                "workspace": {"domain": "test-workspace-1", "id": "T01234N56GB", "name": "test-workspace-1"},
            },
            "id": "9d9b76ce-47bb-4838-a96a-1b5fd4d1b564",
        },
    ),
    RuleTest(
        name="App Resources Added",
        expected_result=True,
        log={
            "action": "app_resources_added",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "entity": {
                "type": "workspace",
                "workspace": {"domain": "test-workspace-1", "id": "T01234N56GB", "name": "test-workspace-1"},
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
    RuleTest(
        name="App Resources Granted",
        expected_result=True,
        log={
            "action": "app_resources_granted",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 16:48:14",
            "details": {
                "export_end_ts": "2022-07-28 09:48:12",
                "export_start_ts": "2022-07-27 09:48:12",
                "export_type": "STANDARD",
            },
            "entity": {
                "type": "workspace",
                "workspace": {"domain": "test-workspace-1", "id": "T01234N56GB", "name": "test-workspace-1"},
            },
            "id": "9d9b76ce-47bb-4838-a96a-1b5fd4d1b564",
        },
    ),
    RuleTest(
        name="Bot Token Upgraded",
        expected_result=True,
        log={
            "action": "bot_token_upgraded",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "entity": {
                "type": "workspace",
                "workspace": {"domain": "test-workspace-1", "id": "T01234N56GB", "name": "test-workspace-1"},
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
    RuleTest(
        name="User Logout",
        expected_result=False,
        log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


@panther_managed
class SlackAuditLogsAppAccessExpanded(Rule):
    id = "Slack.AuditLogs.AppAccessExpanded-prototype"
    display_name = "Slack App Access Expanded"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Privilege Escalation", "Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0004:T1098"]}
    default_severity = Severity.MEDIUM
    default_description = "Detects when a Slack App has had its permission scopes expanded"
    default_reference = "https://slack.com/intl/en-gb/help/articles/1500009181142-Manage-app-settings-and-permissions"
    summary_attributes = ["action", "p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_app_access_expanded_tests
    ACCESS_EXPANDED_ACTIONS = [
        "app_scopes_expanded",
        "app_resources_added",
        "app_resources_granted",
        "bot_token_upgraded",
    ]

    def rule(self, event):
        return event.get("action") in self.ACCESS_EXPANDED_ACTIONS

    def title(self, event):
        return f"Slack App [{deep_get(event, 'entity', 'app', 'name')}] Access Expanded by [{deep_get(event, 'actor', 'user', 'name')}]"

    def alert_context(self, event):
        context = slack_alert_context(event)
        # Diff previous and new scopes
        new_scopes = deep_get(event, "details", "new_scopes", default=[])
        prv_scopes = deep_get(event, "details", "previous_scopes", default=[])
        context["scopes_added"] = [x for x in new_scopes if x not in prv_scopes]
        context["scoped_removed"] = [x for x in prv_scopes if x not in new_scopes]
        return context

    def severity(self, event):
        # Used to escalate to High/Critical if the app is granted admin privileges
        # May want to escalate to "Critical" depending on security posture
        if "admin" in deep_get(event, "entity", "app", "scopes", default=[]):
            return "High"
        # Fallback method in case the admin scope is not directly mentioned in entity for whatever
        if "admin" in deep_get(event, "details", "new_scope", default=[]):
            return "High"
        if "admin" in deep_get(event, "details", "bot_scopes", default=[]):
            return "High"
        return "Medium"



from enum import Enum
from typing import Any

TRUNCATED_STRING_SUFFIX = "... (truncated)"


def try_asdict(item: Any) -> Any:
    if hasattr(item, "asdict"):
        return item.asdict()
    if isinstance(item, list):
        return [try_asdict(v) for v in item]
    if isinstance(item, Enum):
        return item.value
    return item


def truncate(s: str, max_size: int):
    if len(s) > max_size:
        # If generated field exceeds max size, truncate it
        num_characters_to_keep = max_size - len(TRUNCATED_STRING_SUFFIX)
        return s[:num_characters_to_keep] + TRUNCATED_STRING_SUFFIX
    return s


def dedup_list_preserving_order(items: list) -> list:
    s = set(items)
    return [item for item in items if item in s]


def __to_set(value):
    if isinstance(value, str):
        return {value}
    try:
        return set(value)
    except TypeError:
        return {value}


# Get rules based on filter criteria
def filter_iterable_by_kwargs(
    iterable,
    **kwargs,
):
    return [
        x
        for x in iterable
        if all(
            __to_set(getattr(x, key, set())).intersection(__to_set(values))
            for key, values in kwargs.items()
            if values is not None
        )
    ]



import re

from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import aws_rule_context, deep_get
from pypanther.helpers.panther_default import aws_cloudtrail_success

aws_cloud_trail_iam_entity_created_without_cloud_formation_tests: list[RuleTest] = [
    RuleTest(
        name="IAM Entity Created Automatically",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "invokedBy": "cloudformation.amazonaws.com",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/IdentityCFNServiceRole",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateUser",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"userName": "user", "path": "/"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="IAM Entity Created Manually With Approved Role",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/IdentityCFNServiceRole",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateUser",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"userName": "user", "path": "/"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="IAM Entity Created Manually With Approved Role Pattern",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "invokedBy": "cloudformation.amazonaws.com",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::210987654321:role/IdentityCFNServiceRole",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateUser",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"userName": "user", "path": "/"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="IAM Entity Created Manually",
        expected_result=True,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/OtherRole",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateUser",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"userName": "user", "path": "/"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Non IAM Entity Creation Event",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/OtherRole",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "NotCreateUser",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"userName": "user", "path": "/"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
    RuleTest(
        name="Error Manually Creating IAM Entity",
        expected_result=False,
        log={
            "eventVersion": "1.05",
            "errorCode": "EntityAlreadyExists",
            "userIdentity": {
                "type": "AssumedRole",
                "principalId": "1111:tester",
                "arn": "arn:aws:sts::123456789012:assumed-role/tester",
                "accountId": "123456789012",
                "accessKeyId": "1",
                "sessionContext": {
                    "attributes": {"mfaAuthenticated": "true", "creationDate": "2019-01-01T00:00:00Z"},
                    "sessionIssuer": {
                        "type": "Role",
                        "principalId": "1111",
                        "arn": "arn:aws:iam::123456789012:role/OtherRole",
                        "accountId": "123456789012",
                        "userName": "tester",
                    },
                },
            },
            "eventTime": "2019-01-01T00:00:00Z",
            "eventSource": "iam.amazonaws.com",
            "eventName": "CreateUser",
            "awsRegion": "us-east-1",
            "sourceIPAddress": "111.111.111.111",
            "userAgent": "console.amazonaws.com",
            "requestParameters": {"userName": "user", "path": "/"},
            "responseElements": None,
            "requestID": "1",
            "eventID": "1",
            "eventType": "AwsApiCall",
            "recipientAccountId": "123456789012",
        },
    ),
]


@panther_managed
class AWSCloudTrailIAMEntityCreatedWithoutCloudFormation(Rule):
    id = "AWS.CloudTrail.IAMEntityCreatedWithoutCloudFormation-prototype"
    display_name = "IAM Entity Created Without CloudFormation"
    enabled = False
    log_types = [LogType.AWS_CLOUDTRAIL]
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    tags = ["AWS", "Configuration Required", "Identity and Access Management", "Persistence:Create Account"]
    default_severity = Severity.MEDIUM
    default_description = "An IAM Entity (Group, Policy, Role, or User) was created manually. IAM entities should be created in code to ensure that permissions are tracked and managed correctly.\n"
    default_runbook = "Verify whether IAM entity needs to exist. If so, re-create it in an appropriate CloudFormation, Terraform, or other template. Delete the original manually created entity.\n"
    default_reference = "https://blog.awsfundamentals.com/aws-iam-roles-with-aws-cloudformation"
    summary_attributes = ["userAgent", "sourceIpAddress", "recipientAccountId", "p_any_aws_arns"]
    tests = aws_cloud_trail_iam_entity_created_without_cloud_formation_tests
    # The role dedicated for IAM administration
    IAM_ADMIN_ROLES = {"arn:aws:iam::123456789012:role/IdentityCFNServiceRole"}
    # The role patterns dedicated for IAM Service Roles
    IAM_ADMIN_ROLE_PATTERNS = {"arn:aws:iam::[0-9]+:role/IdentityCFNServiceRole"}
    # API calls that are indicative of IAM entity creation
    IAM_ENTITY_CREATION_EVENTS = {
        "BatchCreateUser",
        "CreateGroup",
        "CreateInstanceProfile",
        "CreatePolicy",
        "CreatePolicyVersion",
        "CreateRole",
        "CreateServiceLinkedRole",
        "CreateUser",
    }

    def rule(self, event):
        # Check if this event is in scope
        if not aws_cloudtrail_success(event) or event.get("eventName") not in self.IAM_ENTITY_CREATION_EVENTS:
            return False
        # All IAM changes MUST go through CloudFormation
        if deep_get(event, "userIdentity", "invokedBy") != "cloudformation.amazonaws.com":
            return True
        # Only approved IAM Roles can make IAM Changes
        for admin_role_pattern in self.IAM_ADMIN_ROLE_PATTERNS:
            # Check if the arn matches any role patterns, return False if there is a match
            if (
                len(
                    re.findall(
                        admin_role_pattern,
                        deep_get(event, "userIdentity", "sessionContext", "sessionIssuer", "arn"),
                    ),
                )
                > 0
            ):
                return False
        return deep_get(event, "userIdentity", "sessionContext", "sessionIssuer", "arn") not in self.IAM_ADMIN_ROLES

    def alert_context(self, event):
        return aws_rule_context(event)



import json
from datetime import timedelta

from panther_detection_helpers.caching import put_string_set

import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther import LogType, Rule, RuleMock, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_oss_helpers import resolve_timestamp_string

standard_new_aws_account_created_tests: list[RuleTest] = [
    RuleTest(
        name="AWS Account created",
        expected_result=True,
        mocks=[RuleMock(object_name="put_string_set", return_value="")],
        log={
            "awsRegion": "us-east-1",
            "eventID": "axxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx",
            "eventName": "CreateAccountResult",
            "eventSource": "organizations.amazonaws.com",
            "eventTime": "2021-05-20 15:53:47Z",
            "eventType": "AwsServiceEvent",
            "eventVersion": "1.08",
            "managementEvent": True,
            "p_log_type": "AWS.CloudTrail",
            "p_any_aws_account_ids": ["111111111111", "222222222222"],
            "p_event_time": "2021-05-20 15:53:47Z",
            "readOnly": False,
            "recipientAccountId": "292442345278",
            "serviceEventDetails": '{\n  "createAccountStatus": {\n    "accountId": "1111111111111111",\n    "accountName": "****",\n    "completedTimestamp": "May 20, 2021 3:53:47 PM",\n    "id": "car-aaaaaaaaaaaaaaaaaaaaaaaaaaa",\n    "requestedTimestamp": "May 20, 2021 3:53:44 PM",\n    "state": "SUCCEEDED"\n  }\n}',
        },
    ),
    RuleTest(
        name="Non-Account-Creation Event",
        expected_result=False,
        mocks=[RuleMock(object_name="put_string_set", return_value="")],
        log={
            "awsRegion": "us-east-1",
            "eventName": "CreateAccount",
            "eventTime": "2020-11-05 21:21:46Z",
            "eventType": "AwsApiCall",
            "eventVersion": "1.05",
            "recipientAccountId": "111111111111111111111111111",
            "requestID": "2222222222222222222222222222",
            "requestParameters": '{\n  "accountName": "****",\n  "email": "****",\n  "roleName": "SomeRole"\n}',
            "responseElements": '{\n  "createAccountStatus": {\n    "accountName": "****",\n    "id": "car-3333333333",\n    "requestedTimestamp": "Nov 5, 2020 9:21:45 PM",\n    "state": "IN_PROGRESS"\n  }\n}',
            "sourceIPAddress": "72.177.120.134",
            "p_event_time": "2020-11-05 21:21:46Z",
            "p_log_type": "AWS.CloudTrail",
            "p_any_aws_account_ids": ["222222222222"],
        },
    ),
]


@panther_managed
class StandardNewAWSAccountCreated(Rule):
    id = "Standard.NewAWSAccountCreated-prototype"
    display_name = "New AWS Account Created"
    log_types = [LogType.AWS_CLOUDTRAIL]
    tags = ["DataModel", "Indicator Collection", "Persistence:Create Account"]
    default_severity = Severity.INFO
    reports = {"MITRE ATT&CK": ["TA0003:T1136"]}
    default_description = "A new AWS account was created"
    default_runbook = (
        "A new AWS account was created, ensure it was created through standard practice and is for a valid purpose."
    )
    default_reference = "https://docs.aws.amazon.com/organizations/latest/userguide/orgs_security_incident-response.html#:~:text=AWS%20Organizations%20information%20in%20CloudTrail"
    summary_attributes = ["p_any_aws_account_ids"]
    tests = standard_new_aws_account_created_tests
    # Days an account is considered new
    TTL = timedelta(days=3)

    def parse_new_account_id(self, event):
        if event.get("serviceEventDetails"):
            try:
                details = json.loads(event.get("serviceEventDetails"))
                return str(details.get("createAccountStatus", {}).get("accountId", "<UNKNOWN_ACCOUNT_ID>"))
            except (TypeError, ValueError):
                return "<UNABLE TO PARSE ACCOUNT ID>"
        return "<UNKNOWN ACCOUNT ID>"

    def rule(self, event):
        if event.udm("event_type") != event_type.ACCOUNT_CREATED:
            return False
        account_id = self.parse_new_account_id(event)
        event_time = resolve_timestamp_string(event.get("p_event_time"))
        expiry_time = event_time + self.TTL
        account_event_id = f"new_aws_account_{event.get('p_row_id')}"
        if account_id:
            put_string_set("new_account - " + account_id, [account_event_id], expiry_time.strftime("%s"))
        return True

    def title(self, event):
        return f"A new AWS account has been created. Account ID - [{self.parse_new_account_id(event)}]"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get, deep_walk, okta_alert_context

okta_password_extractionvia_scim_tests: list[RuleTest] = [
    RuleTest(
        name="Other Event",
        expected_result=False,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "user.authentication.auth_via_mfa",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"result": "SUCCESS"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
    RuleTest(
        name="FastPass Phishing Block Event",
        expected_result=True,
        log={
            "actor": {
                "alternateId": "homer.simpson@duff.com",
                "displayName": "Homer Simpson",
                "id": "00abc123",
                "type": "User",
            },
            "authenticationcontext": {"authenticationStep": 0, "externalSessionId": "100-abc-9999"},
            "client": {
                "device": "Computer",
                "geographicalContext": {
                    "city": "Springfield",
                    "country": "United States",
                    "geolocation": {"lat": 20, "lon": -25},
                    "postalCode": "12345",
                    "state": "Ohio",
                },
                "ipAddress": "1.3.2.4",
                "userAgent": {
                    "browser": "CHROME",
                    "os": "Mac OS X",
                    "rawUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/102.0.0.0 Safari/537.36",
                },
                "zone": "null",
            },
            "debugcontext": {
                "debugData": {
                    "requestId": "AbCdEf12G",
                    "requestUri": "/api/v1/users/AbCdEfG/lifecycle/reset_factors",
                    "url": "/api/v1/users/AbCdEfG/lifecycle/reset_factors?",
                },
            },
            "displaymessage": "Authentication of user via MFA",
            "eventtype": "application.lifecycle.update",
            "legacyeventtype": "core.user.factor.attempt_fail",
            "outcome": {"reason": "Pushing user passwords", "result": "SUCCESS"},
            "published": "2022-06-22 18:18:29.015",
            "request": {
                "ipChain": [
                    {
                        "geographicalContext": {
                            "city": "Springfield",
                            "country": "United States",
                            "geolocation": {"lat": 20, "lon": -25},
                            "postalCode": "12345",
                            "state": "Ohio",
                        },
                        "ip": "1.3.2.4",
                        "version": "V4",
                    },
                ],
            },
            "securitycontext": {
                "asNumber": 701,
                "asOrg": "verizon",
                "domain": "verizon.net",
                "isProxy": False,
                "isp": "verizon",
            },
            "severity": "INFO",
            "target": [
                {
                    "alternateId": "peter.griffin@company.com",
                    "displayName": "Peter Griffin",
                    "id": "0002222AAAA",
                    "type": "User",
                },
            ],
            "transaction": {"detail": {}, "id": "ABcDeFgG", "type": "WEB"},
            "uuid": "AbC-123-XyZ",
            "version": "0",
        },
    ),
]


@panther_managed
class OktaPasswordExtractionviaSCIM(Rule):
    id = "Okta.Password.Extraction.via.SCIM-prototype"
    display_name = "Okta Cleartext Passwords Extracted via SCIM Application"
    log_types = [LogType.OKTA_SYSTEM_LOG]
    reports = {"MITRE ATT&CK": ["TA0006:T1556"]}
    default_severity = Severity.HIGH
    default_description = "An application admin has extracted cleartext user passwords via SCIM app. Malcious actors can extract plaintext passwords by creating a SCIM application under their control and configuring it to sync passwords from Okta.\n"
    default_reference = (
        "https://www.authomize.com/blog/authomize-discovers-password-stealing-and-impersonation-risks-to-in-okta/\n"
    )
    dedup_period_minutes = 30
    tests = okta_password_extractionvia_scim_tests

    def rule(self, event):
        return event.get("eventType") == "application.lifecycle.update" and "Pushing user passwords" in deep_get(
            event,
            "outcome",
            "reason",
            default="",
        )

    def title(self, event):
        target = deep_walk(event, "target", "alternateId", default="<alternateId-not-found>", return_val="first")
        return f"{deep_get(event, 'actor', 'displayName', default='<displayName-not-found>')} <{deep_get(event, 'actor', 'alternateId', default='alternateId-not-found')}> extracted cleartext user passwords via SCIM app [{target}]"

    def alert_context(self, event):
        return okta_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

git_hub_user_role_updated_tests: list[RuleTest] = [
    RuleTest(
        name="GitHub - Member Updated",
        expected_result=True,
        log={
            "actor": "cat",
            "action": "org.update_member",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
            "user": "bob",
        },
    ),
    RuleTest(
        name="GitHub - Member Invited",
        expected_result=False,
        log={
            "actor": "cat",
            "action": "org.invite_member",
            "created_at": 1621305118553,
            "p_log_type": "GitHub.Audit",
            "repo": "my-org/my-repo",
            "user": "bob",
        },
    ),
]


@panther_managed
class GitHubUserRoleUpdated(Rule):
    id = "GitHub.User.RoleUpdated-prototype"
    display_name = "GitHub User Role Updated"
    log_types = [LogType.GITHUB_AUDIT]
    tags = ["GitHub", "Persistence:Account Manipulation"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_reference = "https://docs.github.com/en/organizations/managing-peoples-access-to-your-organization-with-roles/roles-in-an-organization"
    default_severity = Severity.HIGH
    default_description = "Detects when a GitHub user role is upgraded to an admin or downgraded to a member"
    tests = git_hub_user_role_updated_tests

    def rule(self, event):
        return event.get("action") == "org.update_member"

    def title(self, event):
        return f"Org owner [{event.udm('actor_user')}] updated user's [{event.get('user')}] role ('admin' or 'member')"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

one_login_user_account_locked_tests: list[RuleTest] = [
    RuleTest(
        name="User account locked via api - first method.",
        expected_result=True,
        log={
            "event_type_id": "531",
            "actor_user_id": 123456,
            "actor_user_name": "Bob Cat",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="User account locked via api - second method.",
        expected_result=True,
        log={
            "event_type_id": "553",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="User account suspended via api.",
        expected_result=True,
        log={
            "event_type_id": "551",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
    RuleTest(
        name="Normal User Activated Event",
        expected_result=False,
        log={
            "event_type_id": "11",
            "actor_user_id": 654321,
            "actor_user_name": "Mountain Lion",
            "user_id": 123456,
            "user_name": "Bob Cat",
        },
    ),
]


@panther_managed
class OneLoginUserAccountLocked(Rule):
    id = "OneLogin.UserAccountLocked-prototype"
    display_name = "OneLogin User Locked"
    log_types = [LogType.ONELOGIN_EVENTS]
    tags = ["OneLogin", "Credential Access:Brute Force"]
    reports = {"MITRE ATT&CK": ["TA0006:T1110"]}
    default_severity = Severity.LOW
    default_description = "User locked or suspended from their account.\n"
    default_reference = "https://onelogin.service-now.com/kb_view_customer.do?sysparm_article=KB0010420"
    default_runbook = "Investigate whether this was caused by expected action.\n"
    summary_attributes = ["account_id", "event_type_id", "user_name", "user_id"]
    tests = one_login_user_account_locked_tests

    def rule(self, event):
        # check for a user locked event
        # event 531 and 553 are user lock events via api
        # event 551 is user suspended via api
        return str(event.get("event_type_id")) in ["531", "553", "551"]

    def title(self, event):
        return f"A user [{event.get('user_name', '<UNKNOWN_USER>')}] was locked or suspended via api call"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import slack_alert_context

slack_audit_logs_ekm_unenrolled_tests: list[RuleTest] = [
    RuleTest(
        name="EKM Unenrolled",
        expected_result=True,
        log={
            "action": "ekm_unenrolled",
            "actor": {
                "type": "user",
                "user": {"email": "user@example.com", "id": "A012B3CDEFG", "name": "username", "team": "T01234N56GB"},
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace",
                    "id": "T01234N56GB",
                    "name": "test-workspace",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
        },
    ),
    RuleTest(
        name="User Logout",
        expected_result=False,
        log={
            "action": "user_logout",
            "actor": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "context": {
                "ip_address": "1.2.3.4",
                "location": {
                    "domain": "test-workspace-1",
                    "id": "T01234N56GB",
                    "name": "test-workspace-1",
                    "type": "workspace",
                },
                "ua": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/103.0.0.0 Safari/537.36",
            },
            "date_create": "2022-07-28 15:22:32",
            "entity": {
                "type": "user",
                "user": {
                    "email": "user@example.com",
                    "id": "W012J3FEWAU",
                    "name": "primary-owner",
                    "team": "T01234N56GB",
                },
            },
            "id": "72cac009-9eb3-4dde-bac6-ee49a32a1789",
        },
    ),
]


@panther_managed
class SlackAuditLogsEKMUnenrolled(Rule):
    id = "Slack.AuditLogs.EKMUnenrolled-prototype"
    display_name = "Slack EKM Unenrolled"
    log_types = [LogType.SLACK_AUDIT_LOGS]
    tags = ["Slack", "Defense Evasion", "Weaken Encryption"]
    reports = {"MITRE ATT&CK": ["TA0005:T1600"]}
    default_severity = Severity.CRITICAL
    default_description = "Detects when a workspace is no longer enrolled or managed by EKM"
    default_reference = "https://slack.com/intl/en-gb/help/articles/360019110974-Slack-Enterprise-Key-Management"
    summary_attributes = ["p_any_ip_addresses", "p_any_emails"]
    tests = slack_audit_logs_ekm_unenrolled_tests

    def rule(self, event):
        # Only alert on the `ekm_unenrolled` action
        return event.get("action") == "ekm_unenrolled"

    def alert_context(self, event):
        return slack_alert_context(event)



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import filter_crowdstrike_fdr_event_type

standard_malicious_ssodns_lookup_tests: list[RuleTest] = [
    RuleTest(
        name="Known Good SSO Domain",
        expected_result=False,
        log={
            "ContextProcessId": "440890253753908704",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2022-08-31 07:03:48.879",
            "DomainName": "company_name_here.okta.com",
            "EffectiveTransmissionClass": 2,
            "Entitlements": "15",
            "RequestType": "1",
            "event_platform": "Mac",
            "event_simpleName": "DnsRequest",
            "name": "DnsRequestMacV2",
            "p_any_domain_names": ["company_name_here.okta.com"],
            "timestamp": "2022-08-31 07:03:49.195",
        },
    ),
    RuleTest(
        name="Potentially Malicious SSO Domain",
        expected_result=True,
        log={
            "ContextProcessId": "440890253753908704",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2022-08-31 07:03:48.879",
            "DomainName": "company_name_here-okta.com",
            "EffectiveTransmissionClass": 2,
            "Entitlements": "15",
            "RequestType": "1",
            "event_platform": "Mac",
            "event_simpleName": "DnsRequest",
            "name": "DnsRequestMacV2",
            "p_any_domain_names": ["company_name_here-okta.com"],
            "timestamp": "2022-08-31 07:03:49.195",
        },
    ),
    RuleTest(
        name="No Domain",
        expected_result=False,
        log={
            "ContextProcessId": "440890253753908704",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2022-08-31 07:03:48.879",
            "EffectiveTransmissionClass": 2,
            "Entitlements": "15",
            "RequestType": "1",
            "event_platform": "Mac",
            "event_simpleName": "DnsRequest",
            "name": "DnsRequestMacV2",
            "p_any_domain_names": [],
            "timestamp": "2022-08-31 07:03:49.195",
        },
    ),
    RuleTest(
        name="Known good and malicious domain",
        expected_result=True,
        log={
            "ContextProcessId": "440890253753908704",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2022-08-31 07:03:48.879",
            "EffectiveTransmissionClass": 2,
            "Entitlements": "15",
            "RequestType": "1",
            "event_platform": "Mac",
            "event_simpleName": "DnsRequest",
            "name": "DnsRequestMacV2",
            "p_any_domain_names": ["company_name_here.okta.com", "company_name_here-maokta.com"],
            "timestamp": "2022-08-31 07:03:49.195",
        },
    ),
    RuleTest(
        name="Known good and malicious domain with Crowdstrike.FDREvent",
        expected_result=True,
        log={
            "event_simpleName": "DnsRequest",
            "name": "DnsRequestMacV1",
            "aid": "00000000000000000000000000000001",
            "aip": "111.111.111.111",
            "cid": "00000000000000000000000000000002",
            "id": "11111111-0000-1111-0000-111111111111",
            "event": {
                "aid": "00000000000000000000000000000001",
                "event_simpleName": "DnsRequest",
                "name": "DnsRequestMacV1",
                "aip": "111.111.111.111",
                "cid": "00000000000000000000000000000002",
                "id": "11111111-0000-1111-0000-111111111111",
                "event_platform": "Mac",
                "timestamp": "2021-10-01 00:00:00.000Z",
                "ConfigBuild": "1007.4.0014301.11",
                "ConfigStateHash": "507116305",
                "Entitlements": "15",
                "ContextThreadId": "0",
                "ContextTimeStamp": "2021-10-08 19:55:04.448Z",
                "ContextProcessId": "111111111111111111",
                "EffectiveTransmissionClass": 2,
                "DomainName": "gooddomain.com",
                "RequestType": "1",
            },
            "event_platform": "Mac",
            "fdr_event_type": "DnsRequest",
            "timestamp": "2021-10-01 00:00:00.000Z",
            "ConfigBuild": "1007.4.0014301.11",
            "ConfigStateHash": "507116305",
            "Entitlements": "15",
            "ContextThreadId": "0",
            "ContextTimeStamp": "2021-10-08 19:55:04.448Z",
            "ContextProcessId": "111111111111111111",
            "EffectiveTransmissionClass": 2,
            "RequestType": "1",
            "p_event_time": "2021-10-08 19:55:04.448Z",
            "p_parse_time": "2021-10-08 20:09:41.933Z",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_row_id": "2ed00000000000000000000000000001",
            "p_source_id": "11111111-1111-1111-1111-111111111111",
            "p_source_label": "Crowdstrike",
            "p_any_ip_addresses": ["111.111.111.111"],
            "p_any_domain_names": ["company_name_here.okta.com", "company_name_here-maokta.com"],
            "p_any_trace_ids": ["00000000000000000000000000000001", "00000000000000000000000000000002"],
        },
    ),
    RuleTest(
        name="non DnsRequest Crowdstrike.FDREvent event",
        expected_result=False,
        log={
            "event_simpleName": "something else",
            "event": {
                "aid": "00000000000000000000000000000001",
                "event_simpleName": "something else",
                "ContextTimeStamp": "2021-10-08 19:55:04.448Z",
                "ContextProcessId": "111111111111111111",
                "EffectiveTransmissionClass": 2,
                "DomainName": "gooddomain.com",
                "RequestType": "1",
            },
            "event_platform": "Mac",
            "fdr_event_type": "something else",
            "ContextProcessId": "111111111111111111",
            "EffectiveTransmissionClass": 2,
            "RequestType": "1",
            "p_event_time": "2021-10-08 19:55:04.448Z",
            "p_parse_time": "2021-10-08 20:09:41.933Z",
            "p_log_type": "Crowdstrike.FDREvent",
            "p_row_id": "2ed00000000000000000000000000001",
            "p_source_id": "11111111-1111-1111-1111-111111111111",
            "p_source_label": "Crowdstrike",
            "p_any_ip_addresses": ["111.111.111.111"],
            "p_any_domain_names": ["company_name_here.okta.com", "company_name_here-maokta.com"],
            "p_any_trace_ids": ["00000000000000000000000000000001", "00000000000000000000000000000002"],
        },
    ),
]


@panther_managed
class StandardMaliciousSSODNSLookup(Rule):
    id = "Standard.MaliciousSSODNSLookup-prototype"
    dedup_period_minutes = 1440
    display_name = "Malicious SSO DNS Lookup"
    enabled = False
    log_types = [
        LogType.CISCO_UMBRELLA_DNS,
        LogType.CROWDSTRIKE_DNS_REQUEST,
        LogType.CROWDSTRIKE_FDR_EVENT,
        LogType.SURICATA_DNS,
        LogType.ZEEK_DNS,
    ]
    default_severity = Severity.MEDIUM
    threshold = 1000
    tags = ["Configuration Required"]
    reports = {"MITRE ATT&CK": ["TA0001:T1566"]}
    default_description = "The rule looks for DNS requests to sites potentially posing as SSO domains."
    default_runbook = "Verify if the destination domain is owned by your organization."
    default_reference = "https://www.cloudns.net/wiki/article/254/#:~:text=A%20DNS%20query%20(also%20known,associated%20with%20a%20domain%20name"
    summary_attributes = ["p_any_ip_addresses"]
    tests = standard_malicious_ssodns_lookup_tests
    '\nWe highly recommend running this logic over 30 days of historical data using data replay\nbefore enabling this in your Panther instance. If ALLOWED_DOMAINS is not fully populated with\ndomains you own, that contain your company name, false positive alerts will be generated.\n\nRecommended steps to enable:\n    1. Change COMPANY_NAME to match your organization\n    2. Update the occurrences of "company_name_here" in malicious_sso_dns_lookup.yml\n    3. Add known domains containing COMPANY_NAME to ALLOWED_DOMAINS\n    4. Run local tests\n    5. Run a Data Replay test to identify unknown domains that should be in ALLOWED_DOMAINS\n'
    # *** Change this to match your company name ***
    COMPANY_NAME = "company_name_here"
    # Ref: https://blog.group-ib.com/0ktapus
    FAKE_KEYWORDS = ["sso", "okta", "corp", "vpn", "citrix", "help", "edge"]
    # Add known good domains that contain your company name
    #   "COMPANY.com",
    ALLOWED_DOMAINS = [".amazonaws.com", ".okta.com", ".oktapreview.com"]

    def rule(self, event):
        # We need to run either for Crowdstrike.DnsRequest or for DnsRequest.FDREvent of 'DnsRequest'
        # type. Crowdstrike.DnsRequest is covered because of the association with the type
        if filter_crowdstrike_fdr_event_type(event, "DnsRequest"):
            return False
        # check domain for company name AND a fake keyword
        for domain in event.get("p_any_domain_names", []):
            domain_was_allowed = [x for x in self.ALLOWED_DOMAINS if domain.lower().endswith(x)]
            if domain_was_allowed:
                continue
            if self.COMPANY_NAME in domain.lower():
                fake_matches = [x for x in self.FAKE_KEYWORDS if x in domain.lower()]
                if fake_matches:
                    return True
        # The domain did not have a fake keyword and the company name
        return False

    def title(self, event):
        return f"Potential Malicious SSO Domain - {event.get('p_any_domain_names', ['NO_DOMAINs_FOUND'])}"



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

gcpiam_custom_role_changes_tests: list[RuleTest] = [
    RuleTest(
        name="Custom Role Created",
        expected_result=True,
        log={
            "protoPayload": {
                "@type": "type.googleapis.com/google.cloud.audit.AuditLog",
                "status": {},
                "authenticationInfo": {
                    "principalEmail": "user.name@runpanther.io",
                    "principalSubject": "user:user.name@runpanther.io",
                },
                "requestMetadata": {
                    "callerIp": "136.24.229.58",
                    "callerSuppliedUserAgent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_4) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/81.0.4044.138 Safari/537.36,gzip(gfe)",
                    "requestAttributes": {"time": "2020-05-15T04:11:28.411897632Z", "auth": {}},
                    "destinationAttributes": {},
                },
                "serviceName": "iam.googleapis.com",
                "methodName": "google.iam.admin.v1.CreateRole",
                "authorizationInfo": [
                    {
                        "resource": "projects/western-verve-123456",
                        "permission": "iam.roles.create",
                        "granted": True,
                        "resourceAttributes": {},
                    },
                ],
                "resourceName": "projects/western-verve-123456/roles/CustomRole",
                "serviceData": {
                    "@type": "type.googleapis.com/google.iam.admin.v1.AuditData",
                    "permissionDelta": {
                        "addedPermissions": [
                            "apigee.apiproducts.create",
                            "apigee.apiproducts.delete",
                            "apigee.apiproducts.get",
                        ],
                    },
                },
                "request": {
                    "parent": "projects/western-verve-123456",
                    "role": {
                        "stage": 1,
                        "included_permissions": [
                            "apigee.apiproducts.create",
                            "apigee.apiproducts.delete",
                            "apigee.apiproducts.get",
                        ],
                        "title": "Jack's custom role",
                        "description": "Created on: 2020-05-14",
                    },
                    "@type": "type.googleapis.com/google.iam.admin.v1.CreateRoleRequest",
                    "role_id": "CustomRole",
                },
                "response": {
                    "name": "projects/western-verve-123456/roles/CustomRole",
                    "group_title": "Custom",
                    "title": "Jack's custom role",
                    "included_permissions": [
                        "apigee.apiproducts.create",
                        "apigee.apiproducts.delete",
                        "apigee.apiproducts.get",
                    ],
                    "@type": "type.googleapis.com/google.iam.admin.v1.Role",
                    "description": "Created on: 2020-05-14",
                    "group_name": "custom",
                    "etag": "BwWlqAHm9IY=",
                    "stage": 1,
                },
            },
            "insertId": "y4nffme2rory",
            "resource": {
                "type": "iam_role",
                "labels": {
                    "project_id": "western-verve-123456",
                    "role_name": "projects/western-verve-123456/roles/CustomRole",
                },
            },
            "timestamp": "2020-05-15T04:11:28.224558457Z",
            "severity": "NOTICE",
            "logName": "projects/western-verve-123456/logs/cloudaudit.googleapis.com%2Factivity",
            "receiveTimestamp": "2020-05-15T04:11:29.472913078Z",
        },
    ),
]


@panther_managed
class GCPIAMCustomRoleChanges(Rule):
    id = "GCP.IAM.CustomRoleChanges-prototype"
    display_name = "GCP IAM Role Has Changed"
    dedup_period_minutes = 1440
    log_types = [LogType.GCP_AUDIT_LOG]
    tags = ["GCP", "Identity & Access Management", "Privilege Escalation:Valid Accounts"]
    reports = {"CIS": ["2.6"], "MITRE ATT&CK": ["TA0004:T1078"]}
    default_severity = Severity.INFO
    default_description = "A custom role has been created, deleted, or updated."
    default_runbook = "No action needed, informational"
    default_reference = "https://cloud.google.com/iam/docs/creating-custom-roles"
    summary_attributes = ["severity", "p_any_ip_addresses", "p_any_domain_names"]
    tests = gcpiam_custom_role_changes_tests
    ROLE_METHODS = {
        "google.iam.admin.v1.CreateRole",
        "google.iam.admin.v1.DeleteRole",
        "google.iam.admin.v1.UpdateRole",
    }

    def rule(self, event):
        return (
            deep_get(event, "resource", "type") == "iam_role"
            and deep_get(event, "protoPayload", "methodName") in self.ROLE_METHODS
        )

    def dedup(self, event):
        return deep_get(event, "resource", "labels", "project_id", default="<UNKNOWN_PROJECT>")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed

carbon_black_alert_v2_passthrough_tests: list[RuleTest] = [
    RuleTest(
        name="True Positive",
        expected_result=True,
        log={
            "p_log_type": "CarbonBlack.AlertV2",
            "alert_notes_present": False,
            "alert_url": "https://d5.carbonblack.net/alerts?orgKey=ABCD1234&s%5Bc%5D%5Bquery_string%5D=id%3A9728f0c8-7810-fb20-15d0-343031500435",
            "attack_tactic": "TA0002",
            "backend_timestamp": "2023-10-04 08:47:36.268000000",
            "backend_update_timestamp": "2023-10-04 08:47:36.268000000",
            "blocked_effective_reputation": "TRUSTED_WHITE_LIST",
            "blocked_name": "c:\\windows\\system32\\cmd.exe",
            "blocked_sha256": "b99d61d874728edc0918ca0eb10eab93d381e7367e377406e65963366c874450",
            "childproc_cmdline": 'C:\\Windows\\system32\\cmd.exe /c ""C:\\Users\\bob.ross\\Downloads\\u0007irreg\\AirRegNCmd.cmd" N18906"',
            "childproc_effective_reputation": "TRUSTED_WHITE_LIST",
            "childproc_guid": "ABCD1234-0a5089d3-00002004-00000000-1d9f69f13af9c8b",
            "childproc_name": "c:\\windows\\system32\\cmd.exe",
            "childproc_sha256": "b99d61d874728edc0918ca0eb10eab93d381e7367e377406e65963366c874450",
            "childproc_username": "DESKTOP-A1B2C3\\bob.ross",
            "detection_timestamp": "2023-10-04 08:46:44.508000000",
            "determination": {"change_timestamp": "2023-10-04 08:47:36.268000000", "value": "NONE"},
            "device_external_ip": "12.34.56.78",
            "device_id": 173050323,
            "device_internal_ip": "192.168.31.23",
            "device_location": "OFFSITE",
            "device_name": "DESKTOP-A1B2C3",
            "device_os": "WINDOWS",
            "device_policy": "Standard",
            "device_policy_id": 390195,
            "device_target_value": "MEDIUM",
            "device_username": "bob.ross@gmail.com",
            "first_event_timestamp": "2023-10-04 08:42:17.182000000",
            "id": "9728f0c8-7810-fb20-15d0-343031500435",
            "is_updated": False,
            "last_event_timestamp": "2023-10-04 08:45:49.524000000",
            "mdr_alert": False,
            "mdr_alert_notes_present": False,
            "org_key": "ABCD1234",
            "parent_effective_reputation": "LOCAL_WHITE",
            "parent_guid": "ABCD1234-0a5089d3-000013f8-00000000-1d9f69f13af9c8b",
            "parent_name": "c:\\windows\\explorer.exe",
            "parent_pid": 5112,
            "parent_reputation": "TRUSTED_WHITE_LIST",
            "parent_sha256": "9c06462b5d1b85517a8ed4b5754c21e6beca5c9a02e42efa8b0e1049431c2972",
            "parent_username": "DESKTOP-A1B2C3\\bob.ross",
            "policy_applied": "APPLIED",
            "primary_event_id": "8e8f7f9f629211ee87ce374372b370f0",
            "process_cmdline": '"C:\\Windows\\System32\\WindowsPowerShell\\u000b1.0\\powershell.exe" ',
            "process_effective_reputation": "TRUSTED_WHITE_LIST",
            "process_guid": "ABCD1234-0a5089d3-000029a8-00000000-1d9f69f13af9c8b",
            "process_issuer": [""],
            "process_md5": "dfd66604ca0898e8e26df7b1635b6326",
            "process_name": "c:\\windows\\system32\\windowspowershell\\u000b1.0\\powershell.exe",
            "process_pid": 10664,
            "process_publisher": [""],
            "process_reputation": "TRUSTED_WHITE_LIST",
            "process_sha256": "d23b67799a0da0143e395ba5db906a22ab08fac4bd5581e275b1e0f1b3fac55c",
            "process_username": "DESKTOP-A1B2C3\\bob.ross",
            "reason": "The application utweb.exe was detected running. A Terminate Policy Action was applied.",
            "reason_code": "T_POL_TERM : utweb.exe",
            "run_state": "RAN",
            "sensor_action": "ALLOW",
            "severity": 3,
            "threat_id": "f1088650549018a6dab7e582a9d3b826",
            "ttps": [
                "POLICY_TERMINATE",
                "NETWORK_ACCESS",
                "MITRE_T1059_003_WIN_CMD_SHELL",
                "ACTIVE_CLIENT",
                "INTERNATIONAL_SITE",
                "MITRE_T1059_001_POWERSHELL",
                "MITRE_T1059_CMD_LINE_OR_SCRIPT_INTER",
                "UNKNOWN_APP",
                "RUN_CMD_SHELL",
            ],
            "type": "CB_ANALYTICS",
            "version": "2.0.0",
            "workflow": {
                "change_timestamp": "2023-10-04 08:47:36.268000000",
                "changed_by": "ALERT_CREATION",
                "changed_by_type": "SYSTEM",
                "closure_reason": "NO_REASON",
                "status": "OPEN",
            },
        },
    ),
    RuleTest(
        name="Alert Updated",
        expected_result=False,
        log={
            "p_log_type": "CarbonBlack.AlertV2",
            "alert_notes_present": False,
            "alert_url": "https://d5.carbonblack.net/alerts?orgKey=ABCD1234&s%5Bc%5D%5Bquery_string%5D=id%3A9728f0c8-7810-fb20-15d0-343031500435",
            "attack_tactic": "TA0002",
            "backend_timestamp": "2023-10-04 08:47:36.268000000",
            "backend_update_timestamp": "2023-10-04 08:47:36.268000000",
            "blocked_effective_reputation": "TRUSTED_WHITE_LIST",
            "blocked_name": "c:\\windows\\system32\\cmd.exe",
            "blocked_sha256": "b99d61d874728edc0918ca0eb10eab93d381e7367e377406e65963366c874450",
            "childproc_cmdline": 'C:\\Windows\\system32\\cmd.exe /c ""C:\\Users\\bob.ross\\Downloads\\u0007irreg\\AirRegNCmd.cmd" N18906"',
            "childproc_effective_reputation": "TRUSTED_WHITE_LIST",
            "childproc_guid": "ABCD1234-0a5089d3-00002004-00000000-1d9f69f13af9c8b",
            "childproc_name": "c:\\windows\\system32\\cmd.exe",
            "childproc_sha256": "b99d61d874728edc0918ca0eb10eab93d381e7367e377406e65963366c874450",
            "childproc_username": "DESKTOP-A1B2C3\\bob.ross",
            "detection_timestamp": "2023-10-04 08:46:44.508000000",
            "determination": {"change_timestamp": "2023-10-04 08:47:36.268000000", "value": "NONE"},
            "device_external_ip": "12.34.56.78",
            "device_id": 173050323,
            "device_internal_ip": "192.168.31.23",
            "device_location": "OFFSITE",
            "device_name": "DESKTOP-A1B2C3",
            "device_os": "WINDOWS",
            "device_policy": "Standard",
            "device_policy_id": 390195,
            "device_target_value": "MEDIUM",
            "device_username": "bob.ross@gmail.com",
            "first_event_timestamp": "2023-10-04 08:42:17.182000000",
            "id": "9728f0c8-7810-fb20-15d0-343031500435",
            "is_updated": False,
            "last_event_timestamp": "2023-10-04 08:45:49.524000000",
            "mdr_alert": False,
            "mdr_alert_notes_present": False,
            "org_key": "ABCD1234",
            "parent_effective_reputation": "LOCAL_WHITE",
            "parent_guid": "ABCD1234-0a5089d3-000013f8-00000000-1d9f69f13af9c8b",
            "parent_name": "c:\\windows\\explorer.exe",
            "parent_pid": 5112,
            "parent_reputation": "TRUSTED_WHITE_LIST",
            "parent_sha256": "9c06462b5d1b85517a8ed4b5754c21e6beca5c9a02e42efa8b0e1049431c2972",
            "parent_username": "DESKTOP-A1B2C3\\bob.ross",
            "policy_applied": "APPLIED",
            "primary_event_id": "8e8f7f9f629211ee87ce374372b370f0",
            "process_cmdline": '"C:\\Windows\\System32\\WindowsPowerShell\\u000b1.0\\powershell.exe" ',
            "process_effective_reputation": "TRUSTED_WHITE_LIST",
            "process_guid": "ABCD1234-0a5089d3-000029a8-00000000-1d9f69f13af9c8b",
            "process_issuer": [""],
            "process_md5": "dfd66604ca0898e8e26df7b1635b6326",
            "process_name": "c:\\windows\\system32\\windowspowershell\\u000b1.0\\powershell.exe",
            "process_pid": 10664,
            "process_publisher": [""],
            "process_reputation": "TRUSTED_WHITE_LIST",
            "process_sha256": "d23b67799a0da0143e395ba5db906a22ab08fac4bd5581e275b1e0f1b3fac55c",
            "process_username": "DESKTOP-A1B2C3\\bob.ross",
            "reason": "The application utweb.exe was detected running. A Terminate Policy Action was applied.",
            "reason_code": "T_POL_TERM : utweb.exe",
            "run_state": "RAN",
            "sensor_action": "ALLOW",
            "severity": 3,
            "threat_id": "f1088650549018a6dab7e582a9d3b826",
            "ttps": [
                "POLICY_TERMINATE",
                "NETWORK_ACCESS",
                "MITRE_T1059_003_WIN_CMD_SHELL",
                "ACTIVE_CLIENT",
                "INTERNATIONAL_SITE",
                "MITRE_T1059_001_POWERSHELL",
                "MITRE_T1059_CMD_LINE_OR_SCRIPT_INTER",
                "UNKNOWN_APP",
                "RUN_CMD_SHELL",
            ],
            "type": "CB_ANALYTICS",
            "version": "2.0.0",
            "workflow": {
                "change_timestamp": "2023-10-04 08:47:36.268000000",
                "changed_by": "admin",
                "changed_by_type": "USER",
                "closure_reason": "RESOLVED",
                "status": "CLOSED",
            },
        },
    ),
]


@panther_managed
class CarbonBlackAlertV2Passthrough(Rule):
    id = "CarbonBlack.AlertV2.Passthrough-prototype"
    default_description = "This rule enriches and contextualizes security alerts generated by Carbon Black.  The alert title and description are dynamically updated based on data included in the alert log."
    display_name = "Carbon Black Passthrough Rule"
    default_runbook = "Review the Carbon Black alert details to determine what malicious behavior was detected, and whether or not it was blocked.  Use the Reference link to view the alert in the Carbon Black console and take remediating actions if necessary."
    default_reference = "https://docs.vmware.com/en/VMware-Carbon-Black-Cloud/services/carbon-black-cloud-user-guide/GUID-0B68199D-6411-45D1-AE0D-2AB9B7A28513.html"
    log_types = [LogType.CARBON_BLACK_ALERT_V2]
    default_severity = Severity.MEDIUM
    dedup_period_minutes = 30
    summary_attributes = [
        "attack_tactic",
        "blocked_name",
        "device_name",
        "device_username",
        "primary_event_id",
        "reason",
        "threat_id",
    ]
    tests = carbon_black_alert_v2_passthrough_tests

    def rule(self, event):
        return event.deep_get("workflow", "changed_by") == "ALERT_CREATION"

    def title(self, event):
        return f"{event.get('attack_tactic', 'CB')}: {event.get('device_username', '<no-user-found>')} on {event.get('device_name', '<no-device-found>')}: {event.get('reason', '<no-reason-found>')}"

    def description(self, event):
        return event.get("reason", "<no-reason-found>")

    def severity(self, event):
        sev = event.get("severity")
        if sev >= 8:
            return "CRITICAL"
        if sev >= 6:
            return "HIGH"
        if sev >= 4:
            return "MEDIUM"
        if sev >= 2:
            return "LOW"
        return "INFO"

    def reference(self, event):
        return event.get("alert_url")

    def dedup(self, event):
        return event.get("id")



from pypanther import LogType, Rule, RuleTest, Severity, panther_managed
from pypanther.helpers.panther_base_helpers import deep_get

g_suite_workspace_gmail_default_routing_rule_modified_tests: list[RuleTest] = [
    RuleTest(
        name="Workspace Admin Creates Default Routing Rule",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "110555555555555555555"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 00:50:03.493000000",
                "uniqueQualifier": "-6333333333333333333",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CREATE_GMAIL_SETTING",
            "parameters": {"SETTING_NAME": "MESSAGE_SECURITY_RULE", "USER_DEFINED_SETTING_NAME": "44444"},
            "type": "EMAIL_SETTINGS",
        },
    ),
    RuleTest(
        name="Workspace Admin Deletes Default Routing Rule",
        expected_result=True,
        log={
            "actor": {"callerType": "USER", "email": "user@example.io", "profileId": "110555555555555555555"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 00:50:41.760000000",
                "uniqueQualifier": "-5015136739334825037",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "DELETE_GMAIL_SETTING",
            "parameters": {"SETTING_NAME": "MESSAGE_SECURITY_RULE", "USER_DEFINED_SETTING_NAME": "44444"},
            "type": "EMAIL_SETTINGS",
        },
    ),
    RuleTest(
        name="Admin Set Default Calendar SHARING_OUTSIDE_DOMAIN Setting to READ_ONLY_ACCESS",
        expected_result=False,
        log={
            "actor": {"callerType": "USER", "email": "example@example.io", "profileId": "12345"},
            "id": {
                "applicationName": "admin",
                "customerId": "D12345",
                "time": "2022-12-11 01:06:26.303000000",
                "uniqueQualifier": "-12345",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "CHANGE_CALENDAR_SETTING",
            "parameters": {
                "DOMAIN_NAME": "example.io",
                "NEW_VALUE": "READ_ONLY_ACCESS",
                "OLD_VALUE": "DEFAULT",
                "ORG_UNIT_NAME": "Example IO",
                "SETTING_NAME": "SHARING_OUTSIDE_DOMAIN",
            },
            "type": "CALENDAR_SETTINGS",
        },
    ),
    RuleTest(
        name="ListObject Type",
        expected_result=False,
        log={
            "actor": {"email": "user@example.io", "profileId": "118111111111111111111"},
            "id": {
                "applicationName": "drive",
                "customerId": "D12345",
                "time": "2022-12-20 17:27:47.080000000",
                "uniqueQualifier": "-7312729053723258069",
            },
            "ipAddress": "12.12.12.12",
            "kind": "admin#reports#activity",
            "name": "rename",
            "parameters": {
                "actor_is_collaborator_account": None,
                "billable": True,
                "doc_id": "1GGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGGG",
                "doc_title": "Document Title- Found Here",
                "doc_type": "presentation",
                "is_encrypted": None,
                "new_value": ["Document Title- Found Here"],
                "old_value": ["Document Title- Old"],
                "owner": "user@example.io",
                "owner_is_shared_drive": None,
                "owner_is_team_drive": None,
                "primary_event": True,
                "visibility": "private",
            },
            "type": "access",
        },
    ),
]


@panther_managed
class GSuiteWorkspaceGmailDefaultRoutingRuleModified(Rule):
    id = "GSuite.Workspace.GmailDefaultRoutingRuleModified-prototype"
    display_name = "GSuite Workspace Gmail Default Routing Rule Modified"
    log_types = [LogType.GSUITE_ACTIVITY_EVENT]
    tags = ["GSuite"]
    reports = {"MITRE ATT&CK": ["TA0003:T1098"]}
    default_severity = Severity.HIGH
    default_description = "A Workspace Admin Has Modified A Default Routing Rule In Gmail\n"
    default_reference = "https://support.google.com/a/answer/2368153?hl=en"
    default_runbook = "Administrators use Default Routing to set up how inbound email is delivered within an organization. The configuration of the default routing rule needs to be inspected in order to verify the intent of the rule is benign.\nIf this change was not planned, inspect the other actions taken by this actor.\n"
    summary_attributes = ["actor:email"]
    tests = g_suite_workspace_gmail_default_routing_rule_modified_tests

    def rule(self, event):
        if all(
            [
                event.get("type", "") == "EMAIL_SETTINGS",
                event.get("name", "").endswith("_GMAIL_SETTING"),
                deep_get(event, "parameters", "SETTING_NAME", default="") == "MESSAGE_SECURITY_RULE",
            ],
        ):
            return True
        return False

    def title(self, event):
        # Gmail records the event name as DELETE_GMAIL_SETTING/CREATE_GMAIL_SETTING
        # We shouldn't be able to enter title() unless event[name] ends with
        #  _GMAIL_SETTING, and as such change_type assumes the happy path.
        change_type = f"{event.get('name', '').split('_')[0].lower()}d"
        return f"GSuite Gmail Default Routing Rule Was [{change_type}] by [{deep_get(event, 'actor', 'email', default='<UNKNOWN_EMAIL>')}]"



import pypanther.helpers.panther_event_type_helpers as event_type
from pypanther.base import DataModel, DataModelMapping, LogType
from pypanther.helpers.panther_base_helpers import ZENDESK_CHANGE_DESCRIPTION, zendesk_get_roles

ZENDESK_TWO_FACTOR_SOURCES = {
    "Two-Factor authentication for all admins and agents",
    "Require Two Factor",
}


def get_event_type(event):
    # user related events
    if event.get("source_type", "") == "user":
        return get_user_event_type(event)
    # account related events
    if event.get("source_type", "") == "account_setting":
        return get_account_setting_event_type(event)
    return None


def get_user_event_type(event):
    # check for login events
    if event.get("action") == "login":
        if event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower().startswith("successful sign-in"):
            return event_type.SUCCESSFUL_LOGIN
    # check for admin assignment
    if event.get("action") == "update":
        _, new_role = zendesk_get_roles(event)
        if new_role and is_admin_role(new_role):
            return event_type.ADMIN_ROLE_ASSIGNED
    return None


def get_account_setting_event_type(event):
    if event.get("source_label", "") in ZENDESK_TWO_FACTOR_SOURCES:
        if event.get(ZENDESK_CHANGE_DESCRIPTION, "").lower() == "disabled":
            return event_type.MFA_DISABLED
    return None


def is_admin_role(new_role):
    if new_role and isinstance(new_role, str):
        for admin in ("admin", "account owner"):
            if admin in new_role.lower():
                return True
    return False


def get_assigned_admin_role(event):
    _, new_role = zendesk_get_roles(event)
    if is_admin_role(new_role):
        return new_role
    return None


def get_user(event):
    # some events will have the user in the source_label field,
    # otherwise the user field may not be relevant
    if event.get("source_type", "").lower() == "user":
        return event.get("source_label")
    return "<UNKNOWN_USER>"


class StandardZendeskAuditLog(DataModel):
    id: str = "Standard.Zendesk.AuditLog"
    display_name: str = "Zendesk Audit"
    enabled: bool = True
    log_types: list[str] = [LogType.ZENDESK_AUDIT]
    mappings: list[DataModelMapping] = [
        DataModelMapping(name="actor_user", path="actor_name"),
        DataModelMapping(name="assigned_admin_role", method=get_assigned_admin_role),
        DataModelMapping(name="event_type", method=get_event_type),
        DataModelMapping(name="source_ip", path="ip_address"),
        DataModelMapping(name="user", method=get_user),
    ]
